"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AITerminalPrompt = exports.KeyboardSelectableList = void 0;
const tslib_1 = require("tslib");
const lodash_1 = require("lodash");
const react_1 = tslib_1.__importStar(require("react"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const terminal_command_suggest_controller_module_less_1 = tslib_1.__importDefault(require("./terminal-command-suggest-controller.module.less"));
// 支持键盘选择的列表
const KeyboardSelectableList = (props) => {
    const { items, handleSuggestionClick, noListen = false, controller } = props;
    // 选中项的索引，默认为最后一个
    const [selectedIndex, setSelectedIndex] = (0, react_1.useState)(-1);
    // 处理键盘事件
    const handleKeyPress = (event) => {
        switch (event.key) {
            case 'ArrowUp': // 上键
                setSelectedIndex((prevIndex) => Math.max(prevIndex - 1, 0));
                break;
            case 'ArrowDown': // 下键
                setSelectedIndex((prevIndex) => Math.min(prevIndex + 1, items.length - 1));
                break;
            case 'Enter': // 回车键
                if (items[selectedIndex]) {
                    handleSuggestionClick(items[selectedIndex].command);
                }
                break;
            default:
                break;
        }
    };
    // 添加全局键盘事件监听器
    (0, react_1.useEffect)(() => {
        if (noListen) {
            return;
        }
        window.addEventListener('keydown', handleKeyPress);
        return () => {
            window.removeEventListener('keydown', handleKeyPress);
        };
    }, [items, selectedIndex]);
    (0, react_1.useEffect)(() => {
        if (!controller) {
            return;
        }
        const disposable = controller.event((e) => {
            if (e === 'ArrowUp') {
                setSelectedIndex((prevIndex) => Math.max(prevIndex - 1, 0));
            }
            if (e === 'ArrowDown' || e === 'Tab') {
                setSelectedIndex((prevIndex) => Math.min(prevIndex + 1, items.length - 1));
            }
            if (e === 'Enter') {
                if (items[selectedIndex]) {
                    handleSuggestionClick(items[selectedIndex].command);
                }
            }
        });
        return () => {
            disposable.dispose();
        };
    }, [controller, selectedIndex, items]);
    (0, react_1.useEffect)(() => {
        // HACK 定位到顶部
        setSelectedIndex(0);
    }, [items]);
    return (react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.suggestions }, items.map((cmd, index) => (react_1.default.createElement("div", { key: index, className: terminal_command_suggest_controller_module_less_1.default.suggestionItem, style: { backgroundColor: index === selectedIndex ? 'var(--selection-background)' : '' }, onClick: () => handleSuggestionClick(cmd.command) },
        react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.suggestionItemContainer },
            react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.suggestionDesc }, cmd.description),
            react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.suggestionCmd }, cmd.command)))))));
};
exports.KeyboardSelectableList = KeyboardSelectableList;
const AITerminalPrompt = (props) => {
    const { onEscTriggered, onSuggestionClick, getAICommandSuggestions, cancelAIRequst } = props;
    const [input, setInput] = (0, react_1.useState)('');
    const [suggestions, setSuggestions] = (0, react_1.useState)([]);
    const [loading, setLoading] = (0, react_1.useState)(false);
    const [statMsg, setStatMsg] = (0, react_1.useState)((0, ide_core_browser_1.localize)('terminal.ai.escClose'));
    const modalRef = (0, react_1.useRef)(null);
    const searchAICommands = async (desc) => {
        setLoading(true);
        setStatMsg((0, ide_core_browser_1.localize)('terminal.ai.requesting'));
        getAICommandSuggestions(desc, (stat) => {
            if (stat === 2) {
                setLoading(false);
                setStatMsg((0, ide_core_browser_1.localize)('terminal.ai.selectHint'));
            }
            if (stat === 1) {
                setLoading(true);
                setStatMsg((0, ide_core_browser_1.localize)('terminal.ai.thinking'));
            }
        }, (suggestions) => {
            setSuggestions([...suggestions].reverse());
        });
    };
    const debouncedSearch = (0, react_1.useCallback)((0, lodash_1.debounce)((desc) => {
        searchAICommands(desc);
    }, 1000), []); // 延迟 1 秒
    // ESC 事件监听
    (0, react_1.useEffect)(() => {
        const handleKeyDown = (event) => {
            if (event.key === 'Escape' || event.keyCode === 27) {
                onEscTriggered();
            }
        };
        document.addEventListener('keydown', handleKeyDown);
        return () => {
            document.removeEventListener('keydown', handleKeyDown);
        };
    }, []);
    // 点击弹框之外的区域关闭弹框
    (0, react_1.useEffect)(() => {
        const handleClickOutside = (event) => {
            if (modalRef.current && !modalRef.current.contains(event.target)) {
                onEscTriggered(); // 调用传入的 onClose 函数来关闭弹框
            }
        };
        // 监听点击事件
        document.addEventListener('mousedown', handleClickOutside);
        // 清理函数
        return () => {
            document.removeEventListener('mousedown', handleClickOutside);
        };
    }, [onEscTriggered]);
    const handleInputChange = (e) => {
        const inputContent = e.target.value;
        setInput(inputContent);
        // 根据输入更新建议列表
        if (inputContent.length > 0 && !inputContent.includes("'")) {
            debouncedSearch(inputContent);
        }
        if (inputContent.length === 0) {
            setSuggestions([]);
            setStatMsg((0, ide_core_browser_1.localize)('terminal.ai.escClose'));
            cancelAIRequst && cancelAIRequst();
        }
    };
    const handleSubmit = (e) => {
        e.preventDefault();
        searchAICommands(input);
    };
    const handleSuggestionClick = (command) => {
        // 处理点击建议命令，这里只是简单地填充输入
        onSuggestionClick(command);
    };
    return (react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.container, ref: modalRef },
        react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.header },
            react_1.default.createElement("div", { style: { flex: '1' } }, (0, ide_core_browser_1.localize)('terminal.ai.headerHint')),
            react_1.default.createElement("div", null,
                react_1.default.createElement(ai_native_1.EnhanceIcon, { className: (0, components_1.getIcon)('close1'), onClick: () => {
                        onEscTriggered();
                    } }))),
        react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.statusContainer }, statMsg),
        react_1.default.createElement("div", { style: { height: '12px' } }),
        react_1.default.createElement("div", { style: { position: 'relative', marginLeft: '-14px' } }, suggestions.length > 0 && (react_1.default.createElement(exports.KeyboardSelectableList, { items: suggestions, handleSuggestionClick: handleSuggestionClick }))),
        react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.inputContainer },
            react_1.default.createElement("form", { onSubmit: handleSubmit, style: { flex: '1' } },
                react_1.default.createElement("input", { type: 'text', value: input, onKeyDown: (event) => {
                        if (event.key === 'ArrowUp' || event.key === 'ArrowDown') {
                            // 阻止默认的键盘上下方向键
                            event.preventDefault();
                        }
                    }, onChange: handleInputChange, placeholder: (0, ide_core_browser_1.localize)('terminal.ai.inputHint'), className: terminal_command_suggest_controller_module_less_1.default.input, autoFocus: true })),
            loading ? (react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.ai_loading },
                react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.loader }),
                react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.loader }),
                react_1.default.createElement("div", { className: terminal_command_suggest_controller_module_less_1.default.loader }))) : (react_1.default.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: terminal_command_suggest_controller_module_less_1.default.send_icon, className: (0, components_1.getIcon)('send-solid'), onClick: () => {
                    searchAICommands(input);
                } })))));
};
exports.AITerminalPrompt = AITerminalPrompt;
//# sourceMappingURL=terminal-command-suggest-controller.js.map