"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AITerminalDecorationService = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const client_1 = require("react-dom/client");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_terminal_next_1 = require("@opensumi/ide-terminal-next");
const inline_chat_feature_registry_1 = require("../../widget/inline-chat/inline-chat.feature.registry");
const terminal_inline_chat_controller_1 = require("../component/terminal-inline-chat-controller");
let AITerminalDecorationService = class AITerminalDecorationService extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.decorationList = [];
        this.decorationRootMap = new Map();
    }
    active() {
        this.disposables.push(this.terminalController.onDidOpenTerminal(({ id }) => this.doSelectionDecoration(id)));
    }
    /**
     * Adds multiline decoration to the terminal.
     */
    addZoneDecoration(terminal, marker, height, inlineWidget) {
        const decoration = terminal.registerDecoration({
            marker,
            width: terminal.cols,
            height,
        });
        if (!decoration) {
            // eslint-disable-next-line no-console
            console.error('Failed to create decoration for line', marker);
            return;
        }
        let root;
        decoration.onRender((element) => {
            if (!root) {
                root = (0, client_1.createRoot)(element);
            }
            // 理论上 React 会确保 DOM 不被重复渲染
            root.render(react_1.default.createElement(terminal_inline_chat_controller_1.TerminalInlineWidgetForDetection, { actions: inlineWidget.operationList, onClickItem: () => {
                    inlineWidget.onClickItem();
                } }));
        });
        decoration.onDispose(() => {
            if (root) {
                root.unmount();
            }
        });
    }
    doSelectionDecoration(clientId) {
        const client = this.terminalController.clients.get(clientId);
        const terminal = client === null || client === void 0 ? void 0 : client.term;
        if (!terminal) {
            return;
        }
        this.addDispose(terminal.onSelectionChange(() => {
            const oldDecoration = this.decorationList.pop();
            if (oldDecoration) {
                (0, ide_core_common_1.runWhenIdle)(() => {
                    oldDecoration === null || oldDecoration === void 0 ? void 0 : oldDecoration.dispose();
                });
            }
            const selection = terminal.getSelectionPosition();
            const selectionTextTrimed = terminal.getSelection().trim();
            if (selection && selectionTextTrimed.length > 0) {
                // 获取选区的右上角位置
                const endRow = selection.end.y;
                const startRow = selection.start.y;
                const cursorY2 = terminal.buffer.active.cursorY + terminal.buffer.active.baseY;
                const cursorYOffset = startRow - cursorY2;
                const selectionHeight = endRow - startRow + 1;
                // 注册一个装饰
                const marker = terminal.registerMarker(cursorYOffset);
                if (marker) {
                    const selectionDecoration = terminal.registerDecoration({
                        marker,
                        width: terminal.cols,
                        height: selectionHeight,
                    });
                    if (selectionDecoration) {
                        let root;
                        selectionDecoration.onRender((element) => {
                            if (!root) {
                                // 创建右上角的 div 元素，用于当 React 容器
                                root = (0, client_1.createRoot)(element);
                            }
                            const allActions = this.inlineChatFeatureRegistry.getTerminalActions();
                            const selectionAction = allActions.filter((action) => { var _a; return ((_a = this.inlineChatFeatureRegistry.getTerminalHandler(action.id)) === null || _a === void 0 ? void 0 : _a.triggerRules) === 'selection'; });
                            root.render(react_1.default.createElement(terminal_inline_chat_controller_1.TerminalInlineWidgetForSelection, { actions: selectionAction, onClickItem: (id) => {
                                    const handler = this.inlineChatFeatureRegistry.getTerminalHandler(id);
                                    if (handler) {
                                        handler.execute(selectionTextTrimed, '');
                                    }
                                } }));
                        });
                        // Decoration dispose 的时候，卸载 React 组件
                        selectionDecoration.onDispose(() => {
                            if (root) {
                                root.unmount();
                            }
                        });
                        // this.selectionDecoration.onDispose()
                        this.decorationList.push(selectionDecoration);
                    }
                }
            }
        }));
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_terminal_next_1.ITerminalController),
    tslib_1.__metadata("design:type", Object)
], AITerminalDecorationService.prototype, "terminalController", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.InlineChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", inline_chat_feature_registry_1.InlineChatFeatureRegistry)
], AITerminalDecorationService.prototype, "inlineChatFeatureRegistry", void 0);
AITerminalDecorationService = tslib_1.__decorate([
    (0, di_1.Injectable)()
], AITerminalDecorationService);
exports.AITerminalDecorationService = AITerminalDecorationService;
//# sourceMappingURL=terminal-decoration.js.map