"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseTerminalDetectionLineMatcher = exports.MatcherType = void 0;
var MatcherType;
(function (MatcherType) {
    MatcherType[MatcherType["base"] = 0] = "base";
    MatcherType[MatcherType["npm"] = 1] = "npm";
    MatcherType[MatcherType["typescript"] = 2] = "typescript";
    MatcherType[MatcherType["node"] = 3] = "node";
    MatcherType[MatcherType["shell"] = 4] = "shell";
    MatcherType[MatcherType["java"] = 5] = "java";
})(MatcherType = exports.MatcherType || (exports.MatcherType = {}));
class BaseTerminalDetectionLineMatcher {
    constructor() {
        this.type = MatcherType.base;
        // 是否多行报错
        this.isMultiLine = false;
        // 记录是否已匹配到错误，配合多行匹配使用
        this.isMatched = false;
        // 如果是多行，可能会存在空行，需要设置非连续报错的间隔
        this.maxSpaceLine = 0;
        // 记录已经匹配到的空行，当前空行数等于maxSpaceLine时，认为是连续报错结束
        this.currentSpaceLine = 0;
    }
    match(output) {
        if (this.doMatch(output)) {
            this.isMatched = true;
            this.currentSpaceLine = 0;
            return true;
        }
        else if (this.isMultiLine && this.isMatched && this.currentSpaceLine < this.maxSpaceLine) {
            this.currentSpaceLine++;
            return true;
        }
        this.isMatched = false;
        return false;
    }
}
exports.BaseTerminalDetectionLineMatcher = BaseTerminalDetectionLineMatcher;
//# sourceMappingURL=base.js.map