"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeMatcher = void 0;
const ansi_parser_1 = require("../utils/ansi-parser");
const base_1 = require("./base");
class NodeMatcher extends base_1.BaseTerminalDetectionLineMatcher {
    constructor() {
        super(...arguments);
        this.type = base_1.MatcherType.node;
        this.isMultiLine = true;
        this.maxSpaceLine = 1;
    }
    /**
     * node 情况
     * 1. 整行都是红色
     * 2. 整行都是红色，但存在其他样式，导致列表数组长度未知
     */
    doMatch(styleList) {
        return this.isSingleLineError(styleList) || this.isMultiPartError(styleList);
    }
    isSingleLineError(styleList) {
        return (styleList.length === 1 &&
            (0, ansi_parser_1.isRedColor)(styleList[0]) &&
            (this.isMatched || !this.isExcludeContent(styleList[0].content)));
    }
    isMultiPartError(styleList) {
        return styleList.length > 1 && !styleList.some((item) => !(0, ansi_parser_1.isRedColor)(item));
    }
    /**
     * 需要排除的一些场景
     * 1. 内容长度小于20
     * 2. 包含 mfsu 以及相关的解释信息 例如： - 不出现三层或以上的 ../ 相对路径查找。、
     */
    isExcludeContent(content) {
        return content.length < 20 || /mfsu/i.test(content) || /\s-\s/.test(content);
    }
}
exports.NodeMatcher = NodeMatcher;
//# sourceMappingURL=node.js.map