"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSCMatcher = void 0;
const ansi_parser_1 = require("../utils/ansi-parser");
const base_1 = require("./base");
class TSCMatcher extends base_1.BaseTerminalDetectionLineMatcher {
    constructor() {
        super(...arguments);
        this.type = base_1.MatcherType.typescript;
        this.isMultiLine = true;
    }
    /**
     * typescript 情况
     * 1. 红色 ERROR
     * 2. TSxxxx
     * 3. Error:
     */
    doMatch(styleList) {
        let hasRedError = false;
        let hasErrorCode = false;
        let hasErrorText = false;
        styleList.forEach((style) => {
            if (/error/i.test(style.content) && (0, ansi_parser_1.isRedColor)(style)) {
                hasRedError = true;
            }
            if (/TS\d+/.test(style.content)) {
                hasErrorCode = true;
            }
            if (/Error:/i.test(style.content)) {
                hasErrorText = true;
            }
        });
        return hasRedError || hasErrorCode || hasErrorText || this.isErrorStack(styleList);
    }
    isErrorStack(styleList) {
        return this.isMatched && !!styleList.find((item) => /^\s+at\s/i.test(item.content));
    }
}
exports.TSCMatcher = TSCMatcher;
//# sourceMappingURL=tsc.js.map