"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isRedColor = exports.ansiParser = exports.ColorCode = exports.StyleCode = void 0;
const tslib_1 = require("tslib");
const ansi_regex_1 = tslib_1.__importDefault(require("ansi-regex"));
const DEFAULT_STYLE = {
    color: 0,
    isSpecialColor: false,
    isRgbColor: false,
    is256Color: false,
    bgColor: 0,
    isSpecialBg: false,
    isRgbBg: false,
    is256Bg: false,
    bold: false,
    underline: false,
};
var StyleCode;
(function (StyleCode) {
    StyleCode[StyleCode["RESET"] = 0] = "RESET";
    StyleCode[StyleCode["BOLD"] = 1] = "BOLD";
    StyleCode[StyleCode["UNDERLINE"] = 4] = "UNDERLINE";
    StyleCode[StyleCode["RESET_BOLD"] = 22] = "RESET_BOLD";
    StyleCode[StyleCode["RESET_UNDERLINE"] = 24] = "RESET_UNDERLINE";
})(StyleCode = exports.StyleCode || (exports.StyleCode = {}));
var ColorCode;
(function (ColorCode) {
    ColorCode[ColorCode["MODE_RGB"] = 2] = "MODE_RGB";
    ColorCode[ColorCode["MODE_256"] = 5] = "MODE_256";
    ColorCode[ColorCode["BLACK"] = 30] = "BLACK";
    ColorCode[ColorCode["RED"] = 31] = "RED";
    ColorCode[ColorCode["GREEN"] = 32] = "GREEN";
    ColorCode[ColorCode["YELLOW"] = 33] = "YELLOW";
    ColorCode[ColorCode["BLUE"] = 34] = "BLUE";
    ColorCode[ColorCode["MAGENTA"] = 35] = "MAGENTA";
    ColorCode[ColorCode["CYAN"] = 36] = "CYAN";
    ColorCode[ColorCode["WHITE"] = 37] = "WHITE";
    ColorCode[ColorCode["SPECIAL_COLOR_MODE"] = 38] = "SPECIAL_COLOR_MODE";
    ColorCode[ColorCode["RESET_COLOR"] = 39] = "RESET_COLOR";
    ColorCode[ColorCode["BLACK_BG"] = 40] = "BLACK_BG";
    ColorCode[ColorCode["RED_BG"] = 41] = "RED_BG";
    ColorCode[ColorCode["GREEN_BG"] = 42] = "GREEN_BG";
    ColorCode[ColorCode["YELLOW_BG"] = 43] = "YELLOW_BG";
    ColorCode[ColorCode["BLUE_BG"] = 44] = "BLUE_BG";
    ColorCode[ColorCode["MAGENTA_BG"] = 45] = "MAGENTA_BG";
    ColorCode[ColorCode["CYAN_BG"] = 46] = "CYAN_BG";
    ColorCode[ColorCode["WHITE_BG"] = 47] = "WHITE_BG";
    ColorCode[ColorCode["SPECIAL_BG_MODE"] = 48] = "SPECIAL_BG_MODE";
    ColorCode[ColorCode["RESET_BG"] = 49] = "RESET_BG";
    ColorCode[ColorCode["BLACK_BRIGHT"] = 90] = "BLACK_BRIGHT";
    ColorCode[ColorCode["RED_BRIGHT"] = 91] = "RED_BRIGHT";
    ColorCode[ColorCode["GREEN_BRIGHT"] = 92] = "GREEN_BRIGHT";
    ColorCode[ColorCode["YELLOW_BRIGHT"] = 93] = "YELLOW_BRIGHT";
    ColorCode[ColorCode["BLUE_BRIGHT"] = 94] = "BLUE_BRIGHT";
    ColorCode[ColorCode["MAGENTA_BRIGHT"] = 95] = "MAGENTA_BRIGHT";
    ColorCode[ColorCode["CYAN_BRIGHT"] = 96] = "CYAN_BRIGHT";
    ColorCode[ColorCode["WHITE_BRIGHT"] = 97] = "WHITE_BRIGHT";
    ColorCode[ColorCode["BLACK_HIGH_INTENSITY"] = 100] = "BLACK_HIGH_INTENSITY";
    ColorCode[ColorCode["RED_HIGH_INTENSITY"] = 101] = "RED_HIGH_INTENSITY";
    ColorCode[ColorCode["GREEN_HIGH_INTENSITY"] = 102] = "GREEN_HIGH_INTENSITY";
    ColorCode[ColorCode["BLUE_HIGH_INTENSITY"] = 104] = "BLUE_HIGH_INTENSITY";
    ColorCode[ColorCode["MAGENTA_HIGH_INTENSITY"] = 105] = "MAGENTA_HIGH_INTENSITY";
    ColorCode[ColorCode["CYAN_HIGH_INTENSITY"] = 106] = "CYAN_HIGH_INTENSITY";
    ColorCode[ColorCode["WHITE_HIGH_INTENSITY"] = 107] = "WHITE_HIGH_INTENSITY";
})(ColorCode = exports.ColorCode || (exports.ColorCode = {}));
function setColorStyle(code, currentStyle, colorType) {
    if (currentStyle.isSpecialColor || currentStyle.isSpecialBg) {
        if (colorType === ColorCode.SPECIAL_COLOR_MODE) {
            currentStyle.color = code;
        }
        else {
            currentStyle.bgColor = code;
        }
    }
    else if ((code >= 30 && code <= 37) || // color
        (code >= 90 && code <= 97) // bright color
    ) {
        currentStyle.color = code;
    }
    else if ((code >= 40 && code <= 47) || // bgColor
        (code >= 100 && code <= 107) // bright bgColor
    ) {
        currentStyle.bgColor = code;
    }
}
function ansiParser(text, style) {
    const regex = (0, ansi_regex_1.default)();
    const parts = [];
    let match;
    let currentPos = 0;
    const currentStyle = Object.assign({}, (style || DEFAULT_STYLE));
    while ((match = regex.exec(text))) {
        const content = text.slice(currentPos, match.index);
        if (content) {
            parts.push(Object.assign({ content }, currentStyle));
        }
        // match style
        const matchStyleCode = match[0].slice(2, -1).split(';').map(Number);
        if (matchStyleCode.length) {
            for (let i = 0; i < matchStyleCode.length; i++) {
                const code = matchStyleCode[i];
                const preCode = matchStyleCode[i - 1];
                switch (code) {
                    case StyleCode.RESET: // reset to default
                        Object.assign(currentStyle, DEFAULT_STYLE);
                        break;
                    case StyleCode.BOLD: // bold
                        currentStyle.bold = true;
                        break;
                    case StyleCode.UNDERLINE: // underline
                        currentStyle.underline = true;
                        break;
                    case StyleCode.RESET_BOLD: // reset bold
                        currentStyle.bold = false;
                        break;
                    case StyleCode.RESET_UNDERLINE: // reset underline
                        currentStyle.underline = false;
                        break;
                    case ColorCode.RESET_COLOR: // reset color
                        currentStyle.color = 0;
                        break;
                    case ColorCode.RESET_BG: // reset bg color
                        currentStyle.bgColor = 0;
                        break;
                    case ColorCode.SPECIAL_COLOR_MODE: // special color mode
                        currentStyle.isSpecialColor = true;
                        break;
                    case ColorCode.SPECIAL_BG_MODE: // special color mode
                        currentStyle.isSpecialBg = true;
                        break;
                    case ColorCode.MODE_RGB: // rgb color mode
                        // e.g: \x1b[38;2;255m preCode is 38
                        if (preCode === ColorCode.SPECIAL_COLOR_MODE) {
                            currentStyle.isRgbColor = true;
                        }
                        else {
                            currentStyle.isRgbBg = true;
                        }
                        break;
                    case ColorCode.MODE_256: // 256 color mode
                        // e.g: \x1b[48;5;255m preCode is 48
                        if (preCode === ColorCode.SPECIAL_COLOR_MODE) {
                            currentStyle.is256Color = true;
                        }
                        else {
                            currentStyle.is256Bg = true;
                        }
                        break;
                    default:
                        setColorStyle(code, currentStyle, matchStyleCode[i - 2]);
                }
            }
        }
        currentPos = match.index + match[0].length;
    }
    const remainingText = text.slice(currentPos);
    if (remainingText) {
        parts.push(Object.assign({ content: remainingText }, currentStyle));
    }
    return {
        parts,
        currentStyle,
    };
}
exports.ansiParser = ansiParser;
function isRedColor(style) {
    const { color, isSpecialColor, is256Color } = style;
    if (isSpecialColor) {
        if (is256Color) {
            return color === 1 || color === 9 || (color >= 160 && color <= 196);
        }
        else {
            return color === 255;
        }
    }
    return color === ColorCode.RED || color === ColorCode.RED_BRIGHT;
}
exports.isRedColor = isRedColor;
//# sourceMappingURL=ansi-parser.js.map