"use strict";
var _ChatManagerService_sessionModels, _ChatManagerService_pendingRequests;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatManagerService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const common_1 = require("../../common");
const chat_model_1 = require("./chat-model");
let ChatManagerService = class ChatManagerService extends ide_core_common_1.Disposable {
    constructor() {
        super();
        _ChatManagerService_sessionModels.set(this, this.registerDispose(new ide_core_common_1.DisposableMap()));
        _ChatManagerService_pendingRequests.set(this, this.registerDispose(new ide_core_common_1.DisposableMap()));
    }
    startSession() {
        const model = this.injector.get(chat_model_1.ChatModel);
        tslib_1.__classPrivateFieldGet(this, _ChatManagerService_sessionModels, "f").set(model.sessionId, model);
        return model;
    }
    getSession(sessionId) {
        return tslib_1.__classPrivateFieldGet(this, _ChatManagerService_sessionModels, "f").get(sessionId);
    }
    clearSession(sessionId) {
        var _a;
        const model = tslib_1.__classPrivateFieldGet(this, _ChatManagerService_sessionModels, "f").get(sessionId);
        if (!model) {
            throw new Error(`Unknown session: ${sessionId}`);
        }
        tslib_1.__classPrivateFieldGet(this, _ChatManagerService_sessionModels, "f").disposeKey(sessionId);
        (_a = tslib_1.__classPrivateFieldGet(this, _ChatManagerService_pendingRequests, "f").get(sessionId)) === null || _a === void 0 ? void 0 : _a.cancel();
        tslib_1.__classPrivateFieldGet(this, _ChatManagerService_pendingRequests, "f").disposeKey(sessionId);
    }
    createRequest(sessionId, message, agentId, command) {
        const model = this.getSession(sessionId);
        if (!model) {
            throw new Error(`Unknown session: ${sessionId}`);
        }
        if (tslib_1.__classPrivateFieldGet(this, _ChatManagerService_pendingRequests, "f").has(sessionId)) {
            return;
        }
        return model.addRequest({ prompt: message, agentId, command });
    }
    async sendRequest(sessionId, request, regenerate) {
        const model = this.getSession(sessionId);
        if (!model) {
            throw new Error(`Unknown session: ${sessionId}`);
        }
        const source = new ide_core_common_1.CancellationTokenSource();
        const token = source.token;
        tslib_1.__classPrivateFieldGet(this, _ChatManagerService_pendingRequests, "f").set(model.sessionId, source);
        const listener = token.onCancellationRequested(() => {
            request.response.cancel();
        });
        const history = [];
        for (const request of model.requests) {
            if (!request.response.isComplete) {
                continue;
            }
            history.push({ role: 1 /* ChatMessageRole.User */, content: request.message.prompt });
            history.push({ role: 2 /* ChatMessageRole.Assistant */, content: request.response.responseText });
        }
        try {
            const progressCallback = (progress) => {
                if (token.isCancellationRequested) {
                    return;
                }
                model.acceptResponseProgress(request, progress);
            };
            const requestProps = {
                sessionId,
                requestId: request.requestId,
                message: request.message.prompt,
                command: request.message.command,
                regenerate,
            };
            const result = await this.chatAgentService.invokeAgent(request.message.agentId, requestProps, progressCallback, history, token);
            if (!token.isCancellationRequested) {
                if (result.errorDetails) {
                    request.response.setErrorDetails(result.errorDetails);
                }
                const followups = this.chatAgentService.getFollowups(request.message.agentId, sessionId, ide_core_common_1.CancellationToken.None);
                followups.then((followups) => {
                    request.response.setFollowups(followups);
                    request.response.complete();
                });
            }
        }
        finally {
            listener.dispose();
            tslib_1.__classPrivateFieldGet(this, _ChatManagerService_pendingRequests, "f").disposeKey(model.sessionId);
        }
    }
    cancelRequest(sessionId) {
        var _a;
        (_a = tslib_1.__classPrivateFieldGet(this, _ChatManagerService_pendingRequests, "f").get(sessionId)) === null || _a === void 0 ? void 0 : _a.cancel();
        tslib_1.__classPrivateFieldGet(this, _ChatManagerService_pendingRequests, "f").disposeKey(sessionId);
    }
};
_ChatManagerService_sessionModels = new WeakMap(), _ChatManagerService_pendingRequests = new WeakMap();
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], ChatManagerService.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.IChatAgentService),
    tslib_1.__metadata("design:type", Object)
], ChatManagerService.prototype, "chatAgentService", void 0);
ChatManagerService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], ChatManagerService);
exports.ChatManagerService = ChatManagerService;
//# sourceMappingURL=chat-manager.service.js.map