import { Disposable, ILogger } from '@opensumi/ide-core-common';
import { IChatAsyncContent, IChatComponent, IChatFollowup, IChatMarkdownContent, IChatModel, IChatProgress, IChatRequestMessage, IChatRequestModel, IChatResponseErrorDetails, IChatTreeData, IChatWelcomeMessageContent, ISampleQuestions } from '../../common';
import { IChatSlashCommandItem } from '../types';
export type IChatProgressResponseContent = IChatMarkdownContent | IChatAsyncContent | IChatTreeData | IChatComponent;
export declare class ChatResponseModel extends Disposable {
    #private;
    readonly session: IChatModel;
    readonly agentId: string;
    get responseParts(): IChatProgressResponseContent[];
    get responseContents(): IChatProgressResponseContent[];
    get isComplete(): boolean;
    get isCanceled(): boolean;
    get requestId(): string;
    get responseText(): string;
    get errorDetails(): IChatResponseErrorDetails | undefined;
    get followups(): IChatFollowup[] | undefined;
    get onDidChange(): import("@opensumi/ide-core-common").Event<void>;
    constructor(requestId: string, session: IChatModel, agentId: string);
    updateContent(progress: IChatProgress, quiet?: boolean): void;
    complete(): void;
    cancel(): void;
    reset(): void;
    setErrorDetails(errorDetails: IChatResponseErrorDetails): void;
    setFollowups(followups: IChatFollowup[]): void;
}
export declare class ChatRequestModel implements IChatRequestModel {
    #private;
    readonly session: IChatModel;
    readonly message: IChatRequestMessage;
    readonly response: ChatResponseModel;
    get requestId(): string;
    constructor(requestId: string, session: IChatModel, message: IChatRequestMessage, response: ChatResponseModel);
}
export declare class ChatModel extends Disposable implements IChatModel {
    #private;
    private static requestIdPool;
    protected readonly logger: ILogger;
    get sessionId(): string;
    get requests(): ChatRequestModel[];
    addRequest(message: IChatRequestMessage): ChatRequestModel;
    acceptResponseProgress(request: ChatRequestModel, progress: IChatProgress, quiet?: boolean): void;
    dispose(): void;
}
export declare class ChatWelcomeMessageModel extends Disposable {
    readonly content: IChatWelcomeMessageContent;
    readonly sampleQuestions: ISampleQuestions[];
    private static nextId;
    private _id;
    get id(): string;
    constructor(content: IChatWelcomeMessageContent, sampleQuestions: ISampleQuestions[]);
}
export declare class ChatSlashCommandItemModel extends Disposable implements IChatSlashCommandItem {
    private readonly chatCommand;
    readonly command?: string | undefined;
    readonly agentId?: string | undefined;
    constructor(chatCommand: IChatSlashCommandItem, command?: string | undefined, agentId?: string | undefined);
    get name(): string;
    get isShortcut(): boolean;
    get icon(): string | undefined;
    get description(): string | undefined;
    get tooltip(): string | undefined;
    get nameWithSlash(): string;
}
//# sourceMappingURL=chat-model.d.ts.map