"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatFeatureRegistry = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const common_1 = require("../../common");
const chat_model_1 = require("./chat-model");
let ChatFeatureRegistry = class ChatFeatureRegistry extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.logger = (0, ide_core_common_1.getDebugLogger)();
        this.slashCommandsMap = new Map();
        this.slashCommandsHandlerMap = new Map();
    }
    dispose() {
        super.dispose();
        this.slashCommandsMap.clear();
        this.slashCommandsHandlerMap.clear();
    }
    registerWelcome(content, sampleQuestions) {
        this.chatWelcomeMessageModel = new chat_model_1.ChatWelcomeMessageModel(content, sampleQuestions);
    }
    registerSlashCommand(command, handler) {
        const { name } = command;
        if (this.slashCommandsMap.has(name)) {
            this.logger.warn(`ChatFeatureRegistry: commands name ${name} already exists`);
            return;
        }
        this.slashCommandsMap.set(name, new chat_model_1.ChatSlashCommandItemModel(command));
        this.slashCommandsHandlerMap.set(name, handler);
    }
    getSlashCommandHandler(name) {
        return this.slashCommandsHandlerMap.get(name);
    }
    getSlashCommand(name) {
        return this.slashCommandsMap.get(name);
    }
    getSlashCommandHandlerBySlashName(slashName) {
        const findCommand = this.getAllSlashCommand().find((item) => item.nameWithSlash === slashName);
        if (!findCommand) {
            return;
        }
        return this.getSlashCommandHandler(findCommand.name);
    }
    getSlashCommandBySlashName(slashName) {
        const findCommand = this.getAllSlashCommand().find((item) => item.nameWithSlash === slashName);
        if (!findCommand) {
            return;
        }
        return this.getSlashCommand(findCommand.name);
    }
    getAllSlashCommand() {
        return Array.from(this.slashCommandsMap.values());
    }
    getAllShortcutSlashCommand() {
        return this.getAllSlashCommand().filter((c) => c.isShortcut === true);
    }
    parseSlashCommand(value) {
        if (value.startsWith(common_1.AI_SLASH)) {
            const allSlashCommands = this.getAllSlashCommand();
            for (const command of allSlashCommands) {
                const { nameWithSlash } = command;
                if (value.startsWith(nameWithSlash)) {
                    return {
                        value: value.slice(nameWithSlash.length),
                        nameWithSlash,
                    };
                }
            }
        }
        return {
            value,
            nameWithSlash: '',
        };
    }
};
ChatFeatureRegistry = tslib_1.__decorate([
    (0, di_1.Injectable)()
], ChatFeatureRegistry);
exports.ChatFeatureRegistry = ChatFeatureRegistry;
//# sourceMappingURL=chat.feature.registry.js.map