"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIChatView = void 0;
const tslib_1 = require("tslib");
const mobx_react_lite_1 = require("mobx-react-lite");
const React = tslib_1.__importStar(require("react"));
const react_chat_elements_1 = require("react-chat-elements");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const command_service_1 = require("@opensumi/ide-editor/lib/browser/monaco-contrib/command/command.service");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
require("react-chat-elements/dist/main.css");
const common_1 = require("../../common");
const ChatEditor_1 = require("../components/ChatEditor");
const ChatInput_1 = require("../components/ChatInput");
const ChatMarkdown_1 = require("../components/ChatMarkdown");
const ChatReply_1 = require("../components/ChatReply");
const ChatThinking_1 = require("../components/ChatThinking");
const SlashCustomRender_1 = require("../components/SlashCustomRender");
const StreamReplyRender_1 = require("../components/StreamReplyRender");
const utils_1 = require("../components/utils");
const WelcomeMsg_1 = require("../components/WelcomeMsg");
const msg_history_manager_1 = require("../model/msg-history-manager");
const msg_stream_manager_1 = require("../model/msg-stream-manager");
const chat_module_less_1 = tslib_1.__importDefault(require("./chat.module.less"));
const SCROLL_CLASSNAME = 'chat_scroll';
exports.AIChatView = (0, mobx_react_lite_1.observer)(() => {
    const aiChatService = (0, ide_core_browser_1.useInjectable)(common_1.IChatInternalService);
    const chatApiService = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatServiceToken);
    const aiReporter = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.IAIReporter);
    const msgStreamManager = (0, ide_core_browser_1.useInjectable)(msg_stream_manager_1.MsgStreamManager);
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const chatRenderRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatRenderRegistryToken);
    const monacoCommandRegistry = (0, ide_core_browser_1.useInjectable)(command_service_1.MonacoCommandRegistry);
    const layoutService = (0, ide_core_browser_1.useInjectable)(ide_main_layout_1.IMainLayoutService);
    const msgHistoryManager = (0, ide_core_browser_1.useInjectable)(msg_history_manager_1.MsgHistoryManager);
    const containerRef = React.useRef(null);
    const chatInputRef = React.useRef(null);
    const [messageListData, dispatchMessage] = React.useReducer((state, action) => {
        switch (action.type) {
            case 'add':
                return [...state, ...(action.payload || [])];
            case 'clear':
                return [];
            case 'init':
                return Array.isArray(action.payload) ? action.payload : [];
            default:
                return state;
        }
    }, []);
    const [loading, setLoading] = React.useState(false);
    const [loading2, setLoading2] = React.useState(false);
    const [agentId, setAgentId] = React.useState('');
    const [defaultAgentId, setDefaultAgentId] = React.useState('');
    const [command, setCommand] = React.useState('');
    const [theme, setTheme] = React.useState(null);
    const aiAssistantName = React.useMemo(() => (0, ide_core_common_1.localize)('aiNative.chat.ai.assistant.name'), []);
    const shortcutCommands = React.useMemo(() => chatFeatureRegistry.getAllShortcutSlashCommand(), [chatFeatureRegistry]);
    const ChatInputWrapperRender = React.useMemo(() => {
        if (chatRenderRegistry.chatInputRender) {
            return chatRenderRegistry.chatInputRender;
        }
        return ChatInput_1.ChatInput;
    }, [chatRenderRegistry.chatInputRender]);
    React.useEffect(() => {
        msgStreamManager.onMsgStatus((event) => {
            if (event === 2 /* EMsgStreamStatus.DONE */ || event === 3 /* EMsgStreamStatus.ERROR */) {
                setLoading2(false);
            }
            else if (event === 1 /* EMsgStreamStatus.THINKING */) {
                setLoading2(true);
            }
            requestAnimationFrame(() => {
                scrollToBottom();
            });
        });
        return () => {
            msgStreamManager.dispose();
        };
    }, []);
    const firstMsg = React.useMemo(() => (0, utils_1.createMessageByAI)({
        id: (0, ide_core_common_1.uuid)(6),
        relationId: '',
        text: React.createElement(WelcomeMsg_1.WelcomeMessage, null),
    }), []);
    const scrollToBottom = React.useCallback(() => {
        if (containerRef && containerRef.current) {
            containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
            // 出现滚动条时出现分割线
            if (containerRef.current.scrollHeight > containerRef.current.clientHeight) {
                containerRef.current.classList.add(SCROLL_CLASSNAME);
            }
        }
    }, [containerRef]);
    React.useEffect(() => {
        dispatchMessage({ type: 'init', payload: [firstMsg] });
    }, []);
    React.useEffect(() => {
        scrollToBottom();
    }, [loading, loading2]);
    React.useEffect(() => {
        const disposer = new ide_core_common_1.Disposable();
        disposer.addDispose(chatApiService.onChatMessageLaunch(async (message) => {
            var _a;
            if (message.immediate !== false) {
                if (loading || loading2) {
                    return;
                }
                await handleSend(message);
            }
            else {
                if (message.agentId) {
                    setAgentId(message.agentId);
                }
                if (message.command) {
                    setCommand(message.command);
                }
                (_a = chatInputRef === null || chatInputRef === void 0 ? void 0 : chatInputRef.current) === null || _a === void 0 ? void 0 : _a.setInputValue(message.message);
            }
        }));
        disposer.addDispose(chatApiService.onChatReplyMessageLaunch((chunk) => {
            const relationId = aiReporter.start(ide_core_common_1.AISerivceType.CustomReplay, {
                msgType: ide_core_common_1.AISerivceType.CustomReplay,
                message: chunk,
            });
            let renderContent = React.createElement(ChatMarkdown_1.ChatMarkdown, { markdown: chunk, fillInIncompleteTokens: true });
            if (chatRenderRegistry.chatAIRoleRender) {
                const ChatAIRoleRender = chatRenderRegistry.chatAIRoleRender;
                renderContent = React.createElement(ChatAIRoleRender, { content: chunk, status: 2 /* EMsgStreamStatus.DONE */ });
            }
            msgHistoryManager.addAssistantMessage({
                content: chunk,
            });
            const aiMessage = (0, utils_1.createMessageByAI)({
                id: (0, ide_core_common_1.uuid)(6),
                relationId,
                text: renderContent,
                className: chat_module_less_1.default.chat_with_more_actions,
            });
            dispatchMessage({ type: 'add', payload: [aiMessage] });
        }));
        return () => disposer.dispose();
    }, [chatApiService, chatRenderRegistry.chatAIRoleRender, msgHistoryManager]);
    React.useEffect(() => {
        const disposer = new ide_core_common_1.Disposable();
        disposer.addDispose(chatAgentService.onDidSendMessage((chunk) => {
            const relationId = aiReporter.start(ide_core_common_1.AISerivceType.Agent, {
                msgType: ide_core_common_1.AISerivceType.Agent,
                message: '',
            });
            msgHistoryManager.addAssistantMessage({
                content: chunk.content,
            });
            const notifyMessage = (0, utils_1.createMessageByAI)({
                id: (0, ide_core_common_1.uuid)(6),
                relationId,
                text: React.createElement(ChatReply_1.ChatNotify, { relationId: relationId, chunk: chunk }),
            }, chat_module_less_1.default.chat_notify);
            dispatchMessage({ type: 'add', payload: [notifyMessage] });
            requestAnimationFrame(() => {
                scrollToBottom();
            });
        }));
        disposer.addDispose(chatAgentService.onDidChangeAgents(async () => {
            const newDefaultAgentId = chatAgentService.getDefaultAgentId();
            setDefaultAgentId(newDefaultAgentId !== null && newDefaultAgentId !== void 0 ? newDefaultAgentId : '');
        }));
        return () => disposer.dispose();
    }, [chatAgentService, msgHistoryManager]);
    const handleSlashCustomRender = React.useCallback(async (value) => {
        const { userMessage, relationId, render, startTime } = value;
        msgHistoryManager.addAssistantMessage({
            type: 'component',
            content: '',
        });
        const aiMessage = (0, utils_1.createMessageByAI)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            className: chat_module_less_1.default.chat_with_more_actions,
            text: (React.createElement(SlashCustomRender_1.SlashCustomRender, { userMessage: userMessage, startTime: startTime, relationId: relationId, renderContent: render })),
        });
        dispatchMessage({ type: 'add', payload: [aiMessage] });
        if (containerRef && containerRef.current) {
            containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
        }
    }, [containerRef, msgHistoryManager]);
    const handleAgentReply = React.useCallback(async (value) => {
        const { message, agentId, command } = value;
        const ChatUserRoleRender = chatRenderRegistry.chatUserRoleRender;
        const request = aiChatService.createRequest(message, agentId, command);
        if (!request) {
            return;
        }
        const startTime = Date.now();
        const relationId = aiReporter.start(ide_core_common_1.AISerivceType.Agent, {
            msgType: ide_core_common_1.AISerivceType.Agent,
            message: value.message,
        });
        msgHistoryManager.addAgentMessage({
            content: message,
            agentId: agentId,
            agentCommand: command,
        });
        const userMessage = (0, utils_1.createMessageByUser)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            text: ChatUserRoleRender ? (React.createElement(ChatUserRoleRender, { content: message, agentId: agentId, command: command })) : (React.createElement(ChatEditor_1.CodeBlockWrapperInput, { relationId: relationId, text: message, agentId: agentId, command: command })),
        }, chat_module_less_1.default.chat_message_code);
        const aiMsgId = msgHistoryManager.addAssistantMessage({
            content: '',
            relationId,
        });
        const aiMessage = (0, utils_1.createMessageByAI)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            className: chat_module_less_1.default.chat_with_more_actions,
            text: (React.createElement(ChatReply_1.ChatReply, { relationId: relationId, request: request, startTime: startTime, onDidChange: (content) => {
                    msgHistoryManager.updateAssistantMessage(aiMsgId, { content });
                }, onRegenerate: () => {
                    msgStreamManager.sendThinkingStatue();
                    aiChatService.sendRequest(request, true);
                } })),
        });
        msgStreamManager.setCurrentSessionId(relationId);
        msgStreamManager.sendThinkingStatue();
        aiChatService.setLatestSessionId(relationId);
        aiChatService.sendRequest(request);
        dispatchMessage({ type: 'add', payload: [userMessage, aiMessage] });
        if (containerRef && containerRef.current) {
            containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
        }
    }, [chatRenderRegistry, chatRenderRegistry.chatUserRoleRender, msgHistoryManager]);
    const handleSend = React.useCallback(async (value) => {
        const { message, prompt, reportType, agentId, command } = value;
        const ChatUserRoleRender = chatRenderRegistry.chatUserRoleRender;
        if (agentId) {
            return handleAgentReply({ message, agentId, command });
        }
        const userInput = {
            type: ide_core_common_1.AISerivceType.Chat,
            message: prompt || message,
        };
        const { nameWithSlash } = chatFeatureRegistry.parseSlashCommand(message);
        let commandHandler;
        if (nameWithSlash) {
            commandHandler = chatFeatureRegistry.getSlashCommandHandlerBySlashName(nameWithSlash);
        }
        if (commandHandler && commandHandler.providerPrompt) {
            const editor = monacoCommandRegistry.getActiveCodeEditor();
            const slashCommandPrompt = await commandHandler.providerPrompt(message, editor);
            userInput.message = slashCommandPrompt;
        }
        const startTime = Date.now();
        const relationId = aiReporter.start(reportType || userInput.type, {
            msgType: reportType || userInput.type,
            message: userInput.message,
        });
        msgHistoryManager.addUserMessage({
            content: message,
        });
        const sendMessage = (0, utils_1.createMessageByUser)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            text: ChatUserRoleRender ? (React.createElement(ChatUserRoleRender, { content: message, agentId: agentId, command: command })) : (React.createElement(ChatEditor_1.CodeBlockWrapperInput, { relationId: relationId, text: message, agentId: agentId, command: command })),
        }, chat_module_less_1.default.chat_message_code);
        dispatchMessage({ type: 'add', payload: [sendMessage] });
        if (commandHandler && commandHandler.providerRender) {
            return handleSlashCustomRender({
                userMessage: message,
                render: commandHandler.providerRender,
                relationId,
                startTime,
            });
        }
        setLoading(true);
        handleReply(userInput, relationId);
    }, [
        messageListData,
        containerRef,
        loading,
        chatFeatureRegistry,
        chatRenderRegistry,
        chatRenderRegistry.chatUserRoleRender,
        msgHistoryManager,
    ]);
    const handleReply = React.useCallback((userInput, relationId) => {
        aiChatService.setLatestSessionId(relationId);
        aiChatService.messageWithStream(userInput.message, {}, relationId);
        const msgId = msgHistoryManager.addAssistantMessage({
            content: '',
            relationId,
        });
        const aiMessage = (0, utils_1.createMessageByAI)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            text: (React.createElement(StreamReplyRender_1.StreamReplyRender, { prompt: userInput.message, relationId: relationId, onDidChange: (content) => {
                    msgHistoryManager.updateAssistantMessage(msgId, { content });
                } })),
            className: chat_module_less_1.default.chat_with_more_actions,
        });
        if (aiMessage) {
            dispatchMessage({ type: 'add', payload: [aiMessage] });
            if (containerRef && containerRef.current) {
                containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
            }
        }
        setLoading(false);
    }, [messageListData, aiChatService, msgHistoryManager]);
    const handleClear = React.useCallback(() => {
        var _a;
        aiChatService.cancelChatViewToken();
        aiChatService.destroyStreamRequest(msgStreamManager.currentSessionId);
        aiChatService.clearSessionModel();
        (_a = containerRef === null || containerRef === void 0 ? void 0 : containerRef.current) === null || _a === void 0 ? void 0 : _a.classList.remove(SCROLL_CLASSNAME);
        dispatchMessage({ type: 'init', payload: [firstMsg] });
    }, [messageListData]);
    const handleThemeClick = (value) => {
        if (loading || loading2) {
            return;
        }
        setTheme(value);
    };
    const handleCloseChatView = React.useCallback(() => {
        layoutService.toggleSlot(common_1.AI_CHAT_VIEW_ID);
    }, [layoutService]);
    return (React.createElement("div", { id: chat_module_less_1.default.ai_chat_view },
        React.createElement("div", { className: chat_module_less_1.default.header_container },
            React.createElement("div", { className: chat_module_less_1.default.left },
                React.createElement("span", { className: chat_module_less_1.default.title }, aiAssistantName)),
            React.createElement("div", { className: chat_module_less_1.default.right },
                React.createElement(components_1.Popover, { insertClass: chat_module_less_1.default.popover_icon, id: 'ai-chat-header-clear', title: (0, ide_core_common_1.localize)('aiNative.operate.clear.title') },
                    React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: chat_module_less_1.default.action_btn, className: (0, ide_core_browser_1.getIcon)('clear'), onClick: handleClear })),
                React.createElement(components_1.Popover, { insertClass: chat_module_less_1.default.popover_icon, id: 'ai-chat-header-close', title: (0, ide_core_common_1.localize)('aiNative.operate.close.title') },
                    React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: chat_module_less_1.default.action_btn, className: (0, ide_core_browser_1.getIcon)('window-close'), onClick: handleCloseChatView })))),
        React.createElement("div", { className: chat_module_less_1.default.body_container },
            React.createElement("div", { className: chat_module_less_1.default.left_bar, id: 'ai_chat_left_container' },
                React.createElement("div", { className: chat_module_less_1.default.chat_container, ref: containerRef },
                    React.createElement(react_chat_elements_1.MessageList, { className: chat_module_less_1.default.message_list, lockable: true, toBottomHeight: '100%', 
                        // @ts-ignore
                        dataSource: messageListData }),
                    loading && (React.createElement("div", { className: chat_module_less_1.default.chat_loading_msg_box },
                        React.createElement(react_chat_elements_1.SystemMessage, { title: aiAssistantName, className: chat_module_less_1.default.smsg, 
                            // @ts-ignore
                            text: React.createElement(ChatThinking_1.ChatThinking, { status: 1 /* EMsgStreamStatus.THINKING */ }) })))),
                React.createElement("div", { className: chat_module_less_1.default.chat_input_wrap },
                    React.createElement("div", { className: chat_module_less_1.default.header_operate },
                        React.createElement("div", { className: chat_module_less_1.default.header_operate_left }, shortcutCommands.map((command) => (React.createElement(components_1.Popover, { id: `ai-chat-shortcut-${command.name}`, title: command.tooltip || command.name },
                            React.createElement("div", { className: chat_module_less_1.default.tag, onClick: () => handleThemeClick(command.nameWithSlash) }, command.name))))),
                        React.createElement("div", { className: chat_module_less_1.default.header_operate_right })),
                    React.createElement(ChatInputWrapperRender, { onSend: (value, agentId, command) => handleSend({ message: value, agentId, command }), disabled: loading || loading2, enableOptions: true, theme: theme, setTheme: setTheme, agentId: agentId, setAgentId: setAgentId, defaultAgentId: defaultAgentId, command: command, setCommand: setCommand, ref: chatInputRef }))))));
});
//# sourceMappingURL=chat.view.js.map