"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatInput = void 0;
const tslib_1 = require("tslib");
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const react_1 = tslib_1.__importStar(require("react"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const command_service_1 = require("@opensumi/ide-editor/lib/browser/monaco-contrib/command/command.service");
const common_1 = require("../../common");
const chat_model_1 = require("../chat/chat-model");
const components_module_less_1 = tslib_1.__importDefault(require("./components.module.less"));
const MAX_WRAPPER_HEIGHT = 160;
const INSTRUCTION_BOTTOM = 8;
const Block = ({ icon, name, description, agentId, command, selectedAgentId, }) => (react_1.default.createElement("div", { className: components_module_less_1.default.block },
    icon && react_1.default.createElement(ai_native_1.EnhanceIcon, { className: icon }),
    name && react_1.default.createElement("span", { className: components_module_less_1.default.name }, name),
    description && react_1.default.createElement("span", { className: components_module_less_1.default.text }, description),
    !selectedAgentId && agentId && command && react_1.default.createElement("span", { className: components_module_less_1.default.agent_label },
        "@",
        agentId)));
const InstructionOptions = ({ onClick, bottom, trigger, agentId: selectedAgentId }) => {
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const agentOptions = (0, react_1.useMemo)(() => {
        if (trigger === '@') {
            return chatAgentService.getAgents().map((a) => new chat_model_1.ChatSlashCommandItemModel({
                icon: '',
                name: `@${a.id} `,
                description: a.metadata.description,
            }, '', a.id));
        }
        else {
            return chatAgentService
                .getCommands()
                .map((c) => new chat_model_1.ChatSlashCommandItemModel({
                icon: '',
                name: `/ ${c.name} `,
                description: c.description,
            }, c.name, c.agentId))
                .filter((item) => !selectedAgentId || item.agentId === selectedAgentId);
        }
    }, [trigger, chatAgentService]);
    const options = (0, react_1.useMemo)(() => {
        if (trigger === '@') {
            return [];
        }
        return chatFeatureRegistry.getAllSlashCommand();
    }, [trigger, chatFeatureRegistry]);
    const handleClick = (0, react_1.useCallback)((name, agentId, command) => {
        if (onClick) {
            onClick(name || '', agentId, command);
        }
    }, [onClick]);
    return (react_1.default.createElement("div", { className: components_module_less_1.default.instruction_options_container, style: { bottom: bottom + 'px' } },
        react_1.default.createElement("div", { className: components_module_less_1.default.options },
            react_1.default.createElement("ul", null, options.concat(agentOptions).map(({ icon, name, nameWithSlash, description, agentId, command }) => (react_1.default.createElement("li", { key: `${agentId || ''}-${name}`, onMouseDown: () => handleClick(nameWithSlash, agentId, command) },
                react_1.default.createElement(Block, { icon: icon, name: nameWithSlash, description: description, agentId: agentId, command: command, selectedAgentId: selectedAgentId }))))))));
};
const ThemeWidget = ({ themeBlock }) => (react_1.default.createElement("div", { className: components_module_less_1.default.theme_container },
    react_1.default.createElement("div", { className: components_module_less_1.default.theme_block }, themeBlock)));
const AgentWidget = ({ agentId, command }) => (react_1.default.createElement("div", { className: components_module_less_1.default.theme_container },
    react_1.default.createElement("div", { className: components_module_less_1.default.theme_block, style: { marginRight: 4 } },
        "@",
        agentId),
    command && react_1.default.createElement("div", { className: components_module_less_1.default.theme_block },
        "/ ",
        command)));
// 指令命令激活组件
exports.ChatInput = react_1.default.forwardRef((props, ref) => {
    const { onSend, onValueChange, enableOptions = false, disabled = false, defaultHeight = 32, autoFocus, setTheme, theme, setAgentId, agentId: propsAgentId, defaultAgentId, setCommand, command, } = props;
    const agentId = propsAgentId || defaultAgentId;
    const [value, setValue] = (0, react_1.useState)(props.value || '');
    const [isShowOptions, setIsShowOptions] = (0, react_1.useState)(false);
    const [wrapperHeight, setWrapperHeight] = (0, react_1.useState)(defaultHeight);
    const textareaRef = (0, react_1.useRef)(null);
    const [focus, setFocus] = (0, react_1.useState)(false);
    const [showExpand, setShowExpand] = (0, react_1.useState)(false);
    const [isExpand, setIsExpand] = (0, react_1.useState)(false);
    const instructionRef = (0, react_1.useRef)(null);
    const [placeholder, setPlaceHolder] = (0, react_1.useState)((0, ide_core_common_1.localize)('aiNative.chat.input.placeholder.default'));
    const monacoCommandRegistry = (0, ide_core_browser_1.useInjectable)(command_service_1.MonacoCommandRegistry);
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const currentAgentIdRef = (0, ide_core_browser_1.useLatest)(agentId);
    (0, react_1.useImperativeHandle)(ref, () => ({
        setInputValue: (v) => {
            setValue(v);
            (0, ide_core_common_1.runWhenIdle)(() => {
                var _a;
                (_a = textareaRef.current) === null || _a === void 0 ? void 0 : _a.focus();
            }, 120);
        },
    }));
    (0, react_1.useEffect)(() => {
        if (props.value !== value) {
            setValue(props.value || '');
        }
    }, [props.value]);
    (0, react_1.useEffect)(() => {
        var _a;
        setTheme(theme || '');
        (_a = textareaRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        const defaultPlaceholder = (0, ide_core_common_1.localize)('aiNative.chat.input.placeholder.default');
        if (!theme) {
            setPlaceHolder(defaultPlaceholder);
            return;
        }
        const findCommandHandler = chatFeatureRegistry.getSlashCommandHandlerBySlashName(theme);
        if (findCommandHandler && findCommandHandler.providerInputPlaceholder) {
            const editor = monacoCommandRegistry.getActiveCodeEditor();
            const placeholder = findCommandHandler.providerInputPlaceholder(value, editor);
            setPlaceHolder(placeholder || defaultPlaceholder);
        }
        else {
            setPlaceHolder(defaultPlaceholder);
        }
    }, [theme, chatFeatureRegistry]);
    (0, react_1.useEffect)(() => {
        acquireOptionsCheck(theme || '');
    }, [theme]);
    (0, react_1.useEffect)(() => {
        var _a;
        if (textareaRef && autoFocus) {
            (_a = textareaRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        }
    }, [textareaRef, autoFocus, props.value]);
    (0, react_1.useEffect)(() => {
        if (enableOptions) {
            if ((value === common_1.AI_SLASH || (value === '@' && chatAgentService.getAgents().length > 0)) && !isExpand) {
                setIsShowOptions(true);
            }
            else {
                setIsShowOptions(false);
            }
        }
        if (value.startsWith(common_1.AI_SLASH)) {
            const { value: newValue, nameWithSlash } = chatFeatureRegistry.parseSlashCommand(value);
            if (nameWithSlash) {
                setValue(newValue);
                setTheme(nameWithSlash);
                return;
            }
        }
        if (chatAgentService.getAgents().length) {
            const parsedInfo = chatAgentService.parseMessage(value, currentAgentIdRef.current);
            if (parsedInfo.agentId || parsedInfo.command) {
                setTheme('');
                setValue(parsedInfo.message);
                if (parsedInfo.agentId) {
                    setAgentId(parsedInfo.agentId);
                }
                if (parsedInfo.command) {
                    setCommand(parsedInfo.command);
                }
            }
        }
        (0, ide_core_common_1.runWhenIdle)(() => {
            // 自适应高度
            if (textareaRef && textareaRef.current && value && !isExpand) {
                textareaRef.current.style.height = 0 + 'px';
                const scrollHeight = textareaRef.current.scrollHeight;
                textareaRef.current.style.height = Math.min(scrollHeight, MAX_WRAPPER_HEIGHT) + 'px';
                const wapperHeight = Math.min(scrollHeight + 12, MAX_WRAPPER_HEIGHT);
                setWrapperHeight(wapperHeight);
                if (wapperHeight > 68) {
                    setShowExpand(true);
                }
                else {
                    setShowExpand(false);
                }
            }
        });
    }, [textareaRef, value, enableOptions, chatFeatureRegistry]);
    (0, react_1.useEffect)(() => {
        if (!value) {
            setWrapperHeight(defaultHeight);
            setShowExpand(false);
            setIsExpand(false);
        }
    }, [value, wrapperHeight]);
    const handleInputChange = (0, react_1.useCallback)((value) => {
        setValue(value);
        if (onValueChange) {
            onValueChange(value);
        }
    }, []);
    const handleSend = (0, react_1.useCallback)(async () => {
        if (disabled) {
            return;
        }
        // 支持 command 不传内容
        if (agentId && (value.trim() || command)) {
            onSend(value, agentId, command);
            setValue('');
            setTheme('');
            setAgentId('');
            setCommand('');
            return;
        }
        if (theme) {
            const chatCommandHandler = chatFeatureRegistry.getSlashCommandHandlerBySlashName(theme);
            if (chatCommandHandler && chatCommandHandler.execute) {
                const editor = monacoCommandRegistry.getActiveCodeEditor();
                await chatCommandHandler.execute(value, (newValue) => {
                    setValue('');
                    onSend(theme + newValue);
                    setTheme('');
                }, editor);
                return;
            }
        }
        else {
            setValue('');
            onSend(value);
            setTheme('');
        }
    }, [onSend, value, agentId, command, chatFeatureRegistry]);
    const acquireOptionsCheck = (0, react_1.useCallback)((themeValue, agentId, command) => {
        // 目前仅 ext 的 command 有 agentId，因此有 agentId，则说明是 ext 注册的
        if (agentId) {
            setIsShowOptions(false);
            setTheme('');
            setAgentId(agentId);
            setCommand(command || '');
            if (textareaRef === null || textareaRef === void 0 ? void 0 : textareaRef.current) {
                const inputValue = textareaRef.current.value;
                if (inputValue === '@' || (command && inputValue === common_1.AI_SLASH)) {
                    setValue('');
                }
                (0, ide_core_common_1.runWhenIdle)(() => textareaRef.current.focus());
            }
        }
        else if (themeValue) {
            setIsShowOptions(false);
            setAgentId('');
            setCommand('');
            const findCommand = chatFeatureRegistry.getSlashCommandBySlashName(themeValue);
            if (findCommand) {
                setTheme(findCommand.nameWithSlash);
            }
            else {
                setTheme('');
            }
            if (textareaRef && textareaRef.current) {
                const inputValue = textareaRef.current.value;
                if (inputValue.length === 1 && inputValue.startsWith(common_1.AI_SLASH)) {
                    setValue('');
                }
                (0, ide_core_common_1.runWhenIdle)(() => textareaRef.current.focus());
            }
        }
    }, [textareaRef, chatFeatureRegistry]);
    const optionsBottomPosition = (0, react_1.useMemo)(() => {
        const customBottom = INSTRUCTION_BOTTOM + wrapperHeight;
        if (isExpand) {
            setIsShowOptions(false);
        }
        return customBottom;
    }, [wrapperHeight]);
    const handleKeyDown = (event) => {
        var _a, _b;
        if (event.key === 'Enter' && !event.nativeEvent.isComposing) {
            if (!event.shiftKey) {
                event.preventDefault();
                handleSend();
            }
        }
        else if (event.key === 'Backspace') {
            if (((_a = textareaRef.current) === null || _a === void 0 ? void 0 : _a.selectionEnd) === 0 && ((_b = textareaRef.current) === null || _b === void 0 ? void 0 : _b.selectionStart) === 0) {
                setTheme('');
                if (agentId) {
                    if (command) {
                        setCommand('');
                    }
                    else {
                        setAgentId('');
                    }
                }
            }
        }
    };
    const handleFocus = () => {
        setFocus(true);
    };
    const handleBlur = (0, react_1.useCallback)(() => {
        setFocus(false);
        setIsShowOptions(false);
    }, [textareaRef]);
    const handleExpandClick = (0, react_1.useCallback)(() => {
        const expand = isExpand;
        setIsExpand(!expand);
        if (!expand) {
            const ele = document.querySelector('#ai_chat_left_container');
            const maxHeight = ele.clientHeight - 68 - (theme ? 32 : 0) - 16;
            setWrapperHeight(maxHeight);
        }
        else {
            setWrapperHeight(defaultHeight);
            setShowExpand(false);
        }
    }, [isExpand]);
    return (react_1.default.createElement("div", { className: (0, classnames_1.default)(components_module_less_1.default.chat_input_container, focus ? components_module_less_1.default.active : null) },
        isShowOptions && (react_1.default.createElement("div", { ref: instructionRef },
            react_1.default.createElement(InstructionOptions, { onClick: acquireOptionsCheck, bottom: optionsBottomPosition, trigger: value, agentId: agentId }))),
        theme && react_1.default.createElement(ThemeWidget, { themeBlock: theme }),
        agentId && react_1.default.createElement(AgentWidget, { agentId: agentId, command: command }),
        showExpand && (react_1.default.createElement("div", { className: components_module_less_1.default.expand_icon, onClick: () => handleExpandClick() },
            react_1.default.createElement(components_1.Popover, { id: 'ai_chat_input_expand', title: (0, ide_core_common_1.localize)(isExpand ? 'aiNative.chat.expand.unfullscreen' : 'aiNative.chat.expand.fullescreen') },
                react_1.default.createElement(components_1.Icon, { className: (0, classnames_1.default)(isExpand ? (0, components_1.getIcon)('unfullscreen') : (0, components_1.getIcon)('fullescreen')) })))),
        react_1.default.createElement(components_1.TextArea, { ref: textareaRef, placeholder: placeholder, wrapperStyle: { height: wrapperHeight + 'px' }, style: {
                height: wrapperHeight - 12 + 2 + 'px',
            }, value: value, onKeyDown: handleKeyDown, onFocus: handleFocus, onBlur: handleBlur, onValueChange: handleInputChange, disabled: disabled, className: components_module_less_1.default.input_wrapper, addonAfter: react_1.default.createElement("div", { className: components_module_less_1.default.input_icon_container },
                react_1.default.createElement("div", { className: (0, classnames_1.default)(components_module_less_1.default.send_chat_btn, focus && components_module_less_1.default.active, disabled && components_module_less_1.default.disabled, props.sendBtnClassName) }, disabled ? (react_1.default.createElement("div", { className: components_module_less_1.default.ai_loading },
                    react_1.default.createElement("div", { className: components_module_less_1.default.loader }),
                    react_1.default.createElement("div", { className: components_module_less_1.default.loader }),
                    react_1.default.createElement("div", { className: components_module_less_1.default.loader }))) : (react_1.default.createElement(components_1.Popover, { id: `ai_chat_input_send_${(0, ide_core_common_1.uuid)(4)}`, title: (0, ide_core_common_1.localize)('aiNative.chat.enter.send'), disable: disabled },
                    react_1.default.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: components_module_less_1.default.send_icon, className: (0, components_1.getIcon)('send-solid'), onClick: () => handleSend() }))))) })));
});
//# sourceMappingURL=ChatInput.js.map