"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatThinkingResult = exports.ChatThinking = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const progress_bar_1 = require("@opensumi/ide-core-browser/lib/progress/progress-bar");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const index_1 = require("../../common/index");
const msg_stream_manager_1 = require("../model/msg-stream-manager");
const components_module_less_1 = tslib_1.__importDefault(require("./components.module.less"));
const ChatThinking = (props) => {
    const { children, status = 1 /* EMsgStreamStatus.THINKING */, message, onStop, showStop = true, hasAgent, thinkingText, } = props;
    const aiChatService = (0, ide_core_browser_1.useInjectable)(index_1.IChatInternalService);
    const chatRenderRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatRenderRegistryToken);
    const msgStreamManager = (0, ide_core_browser_1.useInjectable)(msg_stream_manager_1.MsgStreamManager);
    const CustomThinkingRender = (0, react_1.useMemo)(() => chatRenderRegistry.chatThinkingRender, [chatRenderRegistry, chatRenderRegistry.chatThinkingRender]);
    const handlePause = (0, react_1.useCallback)(async () => {
        // agent 处理有另外的流程
        if (!hasAgent) {
            aiChatService.cancelChatViewToken();
            const { currentSessionId } = msgStreamManager;
            if (currentSessionId) {
                await aiChatService.destroyStreamRequest(currentSessionId);
            }
        }
        onStop && onStop();
    }, [msgStreamManager]);
    const renderContent = (0, react_1.useCallback)(() => {
        if (!children || (status === 1 /* EMsgStreamStatus.THINKING */ && !(message === null || message === void 0 ? void 0 : message.trim()))) {
            if (CustomThinkingRender) {
                return react_1.default.createElement(CustomThinkingRender, { thinkingText: thinkingText });
            }
            return react_1.default.createElement("span", { className: components_module_less_1.default.thinking_text }, thinkingText || 'Thinking...');
        }
        return children;
    }, [status, message, children, thinkingText, CustomThinkingRender]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("div", { className: components_module_less_1.default.content }, renderContent()),
        react_1.default.createElement("div", { className: components_module_less_1.default.thinking_container },
            react_1.default.createElement("div", { className: components_module_less_1.default.stop },
                !CustomThinkingRender && (react_1.default.createElement("span", { className: components_module_less_1.default.progress_bar }, (!!status || !children) && (react_1.default.createElement(progress_bar_1.Progress, { loading: true, wrapperClassName: components_module_less_1.default.ai_native_progress_wrapper })))),
                showStop && (react_1.default.createElement("div", { className: components_module_less_1.default.block, onClick: handlePause },
                    react_1.default.createElement(components_1.Icon, { className: (0, components_1.getIcon)('circle-pause') }),
                    react_1.default.createElement("span", null, (0, ide_core_common_1.localize)('aiNative.operate.stop.title'))))))));
};
exports.ChatThinking = ChatThinking;
const ChatThinkingResult = ({ children, message, status = 2 /* EMsgStreamStatus.DONE */, onRegenerate, sessionId, hasMessage = true, showRegenerate, }) => {
    const aiChatService = (0, ide_core_browser_1.useInjectable)(index_1.IChatInternalService);
    const handleRegenerate = (0, react_1.useCallback)(() => {
        if (onRegenerate) {
            onRegenerate();
        }
    }, [onRegenerate]);
    const renderContent = (0, react_1.useCallback)(() => {
        if ((status === 2 /* EMsgStreamStatus.DONE */ || status === 0 /* EMsgStreamStatus.READY */) && typeof hasMessage === 'boolean'
            ? !hasMessage
            : !(message === null || message === void 0 ? void 0 : message.trim())) {
            return react_1.default.createElement("span", null, (0, ide_core_common_1.localize)('aiNative.chat.stop.immediately'));
        }
        return children;
    }, [status, message, hasMessage, children]);
    const isRenderRegenerate = (0, react_1.useMemo)(() => {
        if ((0, ide_core_common_1.isUndefined)(showRegenerate)) {
            return aiChatService.latestSessionId === sessionId;
        }
        return !!showRegenerate;
    }, [aiChatService.latestSessionId, sessionId, showRegenerate]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("div", { className: components_module_less_1.default.content }, renderContent()),
        react_1.default.createElement("div", { className: components_module_less_1.default.thinking_container },
            react_1.default.createElement("div", { className: components_module_less_1.default.bottom_container },
                react_1.default.createElement("div", { className: components_module_less_1.default.reset }, isRenderRegenerate ? (react_1.default.createElement(ai_native_1.EnhanceIcon, { icon: 'refresh', wrapperClassName: components_module_less_1.default.text_btn, className: components_module_less_1.default.transform, onClick: handleRegenerate },
                    react_1.default.createElement("span", null, (0, ide_core_common_1.localize)('aiNative.operate.afresh.title')))) : null),
                react_1.default.createElement("div", { className: components_module_less_1.default.thumbs },
                    react_1.default.createElement(ai_native_1.Thumbs, { relationId: sessionId, wrapperClassName: components_module_less_1.default.icon_btn }))))));
};
exports.ChatThinkingResult = ChatThinkingResult;
//# sourceMappingURL=ChatThinking.js.map