"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StreamReplyRender = void 0;
const tslib_1 = require("tslib");
const highlight_js_1 = tslib_1.__importDefault(require("highlight.js"));
const react_1 = tslib_1.__importStar(require("react"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const index_1 = require("../../common/index");
const msg_history_manager_1 = require("../model/msg-history-manager");
const msg_stream_manager_1 = require("../model/msg-stream-manager");
const ChatMarkdown_1 = require("./ChatMarkdown");
const ChatThinking_1 = require("./ChatThinking");
const components_module_less_1 = tslib_1.__importDefault(require("./components.module.less"));
const StreamMsgWrapper = (props) => {
    const { sessionId, prompt, startTime = 0, onRegenerate, renderContent } = props;
    const [chunk, setChunk] = react_1.default.useState('');
    const [content, setContent] = react_1.default.useState('');
    const [isError, setIsError] = react_1.default.useState(false);
    const [isDone, setIsDone] = react_1.default.useState(false);
    const [status, setStatus] = react_1.default.useState(1 /* EMsgStreamStatus.THINKING */);
    const msgStreamManager = (0, ide_core_browser_1.useInjectable)(msg_stream_manager_1.MsgStreamManager);
    const aiReporter = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.IAIReporter);
    (0, react_1.useEffect)(() => {
        const disposableCollection = new ide_core_browser_1.DisposableCollection();
        disposableCollection.push(msgStreamManager.onMsgListChange(sessionId)((msg) => {
            if (msg && msgStreamManager.currentSessionId === sessionId) {
                const { delta } = msg;
                setChunk(delta.content);
            }
        }));
        disposableCollection.push(msgStreamManager.onMsgStatus((status) => {
            if (msgStreamManager.currentSessionId === sessionId) {
                setStatus(status);
                setIsError(status === 3 /* EMsgStreamStatus.ERROR */);
                setIsDone(status === 2 /* EMsgStreamStatus.DONE */);
            }
        }));
        highlight_js_1.default.highlightAll();
        return () => disposableCollection.dispose();
    }, [sessionId]);
    const reset = (0, react_1.useCallback)(() => {
        setChunk('');
        setContent('');
        setIsError(false);
        setIsDone(false);
        setStatus(1 /* EMsgStreamStatus.THINKING */);
    }, []);
    (0, react_1.useEffect)(() => {
        if (!chunk) {
            return;
        }
        setContent(content + chunk);
    }, [chunk]);
    const report = (0, react_1.useCallback)((success, stop) => {
        aiReporter.end(sessionId, {
            message: content,
            replytime: Date.now() - startTime,
            success,
            isStop: stop,
        });
    }, [content]);
    (0, react_1.useEffect)(() => {
        if (status === 2 /* EMsgStreamStatus.DONE */) {
            report(true, false);
        }
    }, [status]);
    const handleRegenerate = (0, react_1.useCallback)(() => {
        reset();
        if (onRegenerate) {
            onRegenerate();
        }
    }, [prompt, onRegenerate]);
    const renderMsgList = (0, react_1.useCallback)(() => (react_1.default.createElement("div", { className: components_module_less_1.default.ai_chat_code_wrapper },
        react_1.default.createElement("div", { className: components_module_less_1.default.render_text }, isError ? react_1.default.createElement("span", null, (0, ide_core_common_1.localize)('aiNative.chat.error.response')) : renderContent(content, status)))), [content, isError, isDone, status, sessionId, renderContent]);
    const onStop = () => {
        var _a;
        report(false, true);
        (_a = props.onStop) === null || _a === void 0 ? void 0 : _a.call(props);
    };
    const isThinking = (0, react_1.useMemo)(() => status === 1 /* EMsgStreamStatus.THINKING */ && msgStreamManager.currentSessionId === sessionId, [status, sessionId, msgStreamManager.currentSessionId]);
    if (isThinking) {
        return (react_1.default.createElement(ChatThinking_1.ChatThinking, { status: status, message: content, onStop: onStop }, renderMsgList()));
    }
    return (react_1.default.createElement(ChatThinking_1.ChatThinkingResult, { status: status, message: content, onRegenerate: handleRegenerate, sessionId: sessionId }, renderMsgList()));
};
// 流式输出渲染组件
const StreamReplyRender = (props) => {
    const { prompt, relationId, onDidChange } = props;
    const aiChatService = (0, ide_core_browser_1.useInjectable)(index_1.IChatInternalService);
    const chatRenderRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatRenderRegistryToken);
    const msgHistoryManager = (0, ide_core_browser_1.useInjectable)(msg_history_manager_1.MsgHistoryManager);
    const send = react_1.default.useCallback((isRetry = false) => {
        aiChatService.setLatestSessionId(relationId);
        aiChatService.messageWithStream(prompt, isRetry ? { enableGptCache: false } : {}, relationId);
    }, [aiChatService, relationId, prompt]);
    const renderContent = (0, react_1.useCallback)((content, status) => {
        if (onDidChange) {
            onDidChange(content);
        }
        if (chatRenderRegistry.chatAIRoleRender) {
            const ChatAIRoleRender = chatRenderRegistry.chatAIRoleRender;
            return react_1.default.createElement(ChatAIRoleRender, { status: status, content: content });
        }
        return react_1.default.createElement(ChatMarkdown_1.ChatMarkdown, { markdown: content, fillInIncompleteTokens: true });
    }, [chatRenderRegistry, chatRenderRegistry.chatAIRoleRender, msgHistoryManager, onDidChange]);
    return (react_1.default.createElement(StreamMsgWrapper, { sessionId: relationId, prompt: prompt, onRegenerate: () => send(true), renderContent: (content, status) => renderContent(content, status) }));
};
exports.StreamReplyRender = StreamReplyRender;
//# sourceMappingURL=StreamReplyRender.js.map