"use strict";
var AICompletionsService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AICompletionsService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const reporter_1 = require("@opensumi/ide-core-common/lib/types/ai-native/reporter");
let AICompletionsService = AICompletionsService_1 = class AICompletionsService extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.cancelIndicator = new ide_core_common_1.CancellationTokenSource();
        // 是否使用默认的补全模型
        this.isDefaultCompletionModel = true;
        // 是否显示了 inline 补全
        this.isVisibleCompletion = false;
    }
    recordRenderTime() {
        this.lastRenderTime = Date.now();
    }
    recordCompletionUseTime(preTime) {
        this.lastCompletionUseTime = Date.now() - preTime;
    }
    async complete(data, model, position, token) {
        const doCompletion = async (data) => {
            try {
                this.isDefaultCompletionModel = true;
                const now = Date.now();
                const result = (await this.aiBackService.requestCompletion(data, this.cancelIndicator.token));
                this.recordCompletionUseTime(now);
                return result;
            }
            catch (error) {
                return null;
            }
        };
        if (this.lastMiddlewareInlineCompletion) {
            this.isDefaultCompletionModel = false;
            return this.lastMiddlewareInlineCompletion(model, position, token, doCompletion, data);
        }
        return doCompletion(data);
    }
    setMiddlewareComplete(provideInlineCompletions) {
        this.lastMiddlewareInlineCompletion = provideInlineCompletions;
    }
    async report(data) {
        const { relationId, accept } = data;
        data.renderingTime = Date.now() - this.lastRenderTime;
        data.completionUseTime = this.lastCompletionUseTime;
        this.aiBackService.reportCompletion(data);
        this.reporterEnd(relationId, { success: true, isReceive: accept, renderingTime: data.renderingTime });
        this.isVisibleCompletion = false;
    }
    async reporterEnd(relationId, data) {
        this.aiReporter.end(relationId, Object.assign(Object.assign({}, data), { isValid: typeof data.renderingTime === 'number' ? data.renderingTime > 750 : false }));
    }
    setVisibleCompletion(visible) {
        // 如果之前是 true，现在是 false，说明并没有进行采纳
        if (this.isVisibleCompletion === true && visible === false) {
            this.report({ sessionId: this.lastSessionId, accept: false, relationId: this.lastRelationId });
        }
        if (visible === true) {
            this.isVisibleCompletion = visible;
            this.recordRenderTime();
        }
        else {
            this.isVisibleCompletion = false;
        }
    }
    setLastSessionId(sessionId) {
        this.lastSessionId = sessionId;
    }
    setLastRelationId(relationId) {
        this.lastRelationId = relationId;
    }
    async cancelRequest() {
        this.cancelIndicator.cancel();
        this.cancelIndicator = new ide_core_common_1.CancellationTokenSource();
    }
    updateStatusBarItem(content, isLoading) {
        const text = isLoading ? `$(loading~spin) ${content}` : `$(magic-wand) ${content}`;
        this.statusBarService.addElement(AICompletionsService_1.STATUS_ID, {
            text,
            alignment: ide_core_browser_1.StatusBarAlignment.RIGHT,
            priority: 1,
        });
    }
    hideStatusBarItem() {
        this.statusBarService.removeElement(AICompletionsService_1.STATUS_ID);
    }
};
AICompletionsService.STATUS_ID = 'ai_completion_status';
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.AIBackSerivcePath),
    tslib_1.__metadata("design:type", Object)
], AICompletionsService.prototype, "aiBackService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IStatusBarService),
    tslib_1.__metadata("design:type", Object)
], AICompletionsService.prototype, "statusBarService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(reporter_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], AICompletionsService.prototype, "aiReporter", void 0);
AICompletionsService = AICompletionsService_1 = tslib_1.__decorate([
    (0, di_1.Injectable)()
], AICompletionsService);
exports.AICompletionsService = AICompletionsService;
//# sourceMappingURL=ai-completions.service.js.map