"use strict";
// 所有已支持语言中代码块的 AST 节点类型
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeSitterLanguageFacts = exports.knownLanguageFacts = void 0;
const tslib_1 = require("tslib");
tslib_1.__exportStar(require("./types"), exports);
const di_1 = require("@opensumi/di");
const golang_1 = require("./golang");
const java_1 = require("./java");
const javascript_1 = require("./javascript");
const javascriptreact_1 = require("./javascriptreact");
const python_1 = require("./python");
const rust_1 = require("./rust");
const typescript_1 = require("./typescript");
const typescriptreact_1 = require("./typescriptreact");
const emptySet = new Set();
exports.knownLanguageFacts = [
    golang_1.GolangLanguageFacts,
    java_1.JavaLanguageFacts,
    javascript_1.JavaScriptLanguageFacts,
    javascriptreact_1.JavaScriptReactLanguageFacts,
    python_1.PythonLanguageFacts,
    rust_1.RustLanguageFacts,
    typescript_1.TypeScriptLanguageFacts,
    typescriptreact_1.TypeScriptReactLanguageFacts,
];
let TreeSitterLanguageFacts = class TreeSitterLanguageFacts {
    constructor() {
        this.langs = new Map();
        exports.knownLanguageFacts.forEach((Fact) => {
            const fact = new Fact();
            this.langs.set(fact.name, fact);
        });
    }
    isCodeBlock(language, type) {
        const languageFacts = this.langs.get(language);
        if (languageFacts && languageFacts.isCodeBlock) {
            return languageFacts.isCodeBlock(type);
        }
        return false;
    }
    isFunctionCodeBlock(language, type) {
        const languageFacts = this.langs.get(language);
        if (languageFacts && languageFacts.isFunctionCodeBlocks) {
            return languageFacts.isFunctionCodeBlocks(type);
        }
        return false;
    }
    provideFunctionInfo(language, node) {
        const languageFacts = this.langs.get(language);
        if (languageFacts && languageFacts.provideFunctionInfo) {
            return languageFacts.provideFunctionInfo(node);
        }
        return null;
    }
    getCodeBlockTypes(language) {
        const languageFacts = this.langs.get(language);
        if (languageFacts) {
            return languageFacts.provideCodeBlocks();
        }
        return emptySet;
    }
};
TreeSitterLanguageFacts = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], TreeSitterLanguageFacts);
exports.TreeSitterLanguageFacts = TreeSitterLanguageFacts;
//# sourceMappingURL=index.js.map