"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaScriptLanguageFacts = exports.functionBlockCodeTypes = exports.javascriptBlockCodeTypes = void 0;
const common_1 = require("../common");
/**
 * javascript 中表示代码块的节点类型
 */
exports.javascriptBlockCodeTypes = [
    'function',
    'function_declaration',
    'function_expression',
    'arrow_function',
    'class_declaration',
    'method_definition',
    'if_statement',
    'switch_case',
    'while_statement',
    'do_statement',
    'for_statement',
    'for_in_statement',
    'try_statement',
    'catch_clause',
    'block',
    'export_statement',
    'lexical_declaration',
];
exports.functionBlockCodeTypes = [
    'function',
    'function_declaration',
    'function_expression',
    'arrow_function',
    'method_definition',
];
const functionBlockSet = new Set(exports.functionBlockCodeTypes);
const blockSet = new Set(exports.javascriptBlockCodeTypes);
class JavaScriptLanguageFacts {
    constructor() {
        this.name = 'javascript';
        this.listCommentStyle = '// ';
        this.blockCommentStyle = {
            start: '/**',
            end: ' */',
            linePrefix: ' * ',
        };
    }
    provideCodeBlocks() {
        return blockSet;
    }
    isCodeBlock(type) {
        return blockSet.has(type);
    }
    isFunctionCodeBlocks(type) {
        return functionBlockSet.has(type);
    }
    provideFunctionInfo(node) {
        var _a, _b, _c;
        switch (node.type) {
            case 'function_declaration':
            case 'function_expression':
                return {
                    infoCategory: 'function',
                    type: node.type,
                    name: ((_a = node.firstNamedChild) === null || _a === void 0 ? void 0 : _a.text) || '',
                    signatures: node.children
                        .filter((child) => child.type === 'parameter')
                        .map((param) => { var _a; return ((_a = param.firstChild) === null || _a === void 0 ? void 0 : _a.text) || ''; }),
                    range: (0, common_1.toMonacoRange)(node),
                };
            case 'arrow_function': {
                const parent = node.parent;
                // example: const a = () => {}
                if (parent &&
                    parent.type === 'variable_declarator' &&
                    parent.parent &&
                    (parent.parent.type === 'lexical_declaration' ||
                        parent.parent.type === 'export_statement' ||
                        parent.parent.type === 'variable_declaration')) {
                    return {
                        infoCategory: 'function',
                        type: node.type,
                        name: ((_b = parent.firstChild) === null || _b === void 0 ? void 0 : _b.text) || '',
                        signatures: node.children
                            .filter((child) => child.type === 'parameter')
                            .map((param) => { var _a; return ((_a = param.firstChild) === null || _a === void 0 ? void 0 : _a.text) || ''; }),
                        range: (0, common_1.toMonacoRange)(parent.parent),
                    };
                }
                return null;
            }
            case 'method_definition':
                return {
                    infoCategory: 'function',
                    type: node.type,
                    name: ((_c = node.firstNamedChild) === null || _c === void 0 ? void 0 : _c.text) || '',
                    signatures: node.children
                        .filter((child) => child.type === 'parameter')
                        .map((param) => { var _a; return ((_a = param.firstChild) === null || _a === void 0 ? void 0 : _a.text) || ''; }),
                    range: (0, common_1.toMonacoRange)(node),
                };
        }
        return null;
    }
}
exports.JavaScriptLanguageFacts = JavaScriptLanguageFacts;
//# sourceMappingURL=javascript.js.map