"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WasmModuleManager = void 0;
const tslib_1 = require("tslib");
const web_tree_sitter_1 = tslib_1.__importDefault(require("web-tree-sitter"));
const di_1 = require("@opensumi/di");
const types_1 = require("@opensumi/ide-core-browser/lib/application/runtime/types");
const ide_utils_1 = require("@opensumi/ide-utils");
/**
 * Managing and caching the wasm module
 */
let WasmModuleManager = class WasmModuleManager {
    async resolveResourceUri() {
        const uri = await this.rendererRuntime.provideResourceUri(types_1.EKnownResources.TreeSitterWasmDirectory);
        this.resolvedResourceUriDeferred.resolve(uri);
    }
    constructor() {
        this.resolvedResourceUriDeferred = new ide_utils_1.Deferred();
        this.cachedRuntime = new Map();
        this.parserInitialized = false;
        this.resolveResourceUri();
    }
    async initParser() {
        const baseUrl = await this.resolvedResourceUriDeferred.promise;
        const wasmPath = `${baseUrl}/tree-sitter.wasm`;
        if (!this.parserInitialized) {
            await web_tree_sitter_1.default.init({
                locateFile: () => wasmPath,
            });
            this.parserInitialized = true;
        }
        return new web_tree_sitter_1.default();
    }
    async loadLanguage(language) {
        if (!this.cachedRuntime.has(language)) {
            const deferred = new ide_utils_1.Deferred();
            this.cachedRuntime.set(language, deferred);
            const baseUrl = await this.resolvedResourceUriDeferred.promise;
            const wasmUrl = `${baseUrl}/tree-sitter-${language}.wasm`;
            fetch(wasmUrl)
                .then((res) => res.arrayBuffer())
                .then((buffer) => {
                deferred.resolve(buffer);
            });
        }
        return this.cachedRuntime.get(language).promise;
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.IRendererRuntime),
    tslib_1.__metadata("design:type", Object)
], WasmModuleManager.prototype, "rendererRuntime", void 0);
WasmModuleManager = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], WasmModuleManager);
exports.WasmModuleManager = WasmModuleManager;
//# sourceMappingURL=wasm-manager.js.map