"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MergeConflictContribution = exports.MERGE_CONFLICT = void 0;
const tslib_1 = require("tslib");
const debounce_1 = tslib_1.__importDefault(require("lodash/debounce"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const conflict_report_service_1 = require("@opensumi/ide-core-browser/lib/ai-native/conflict-report.service");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const browser_1 = require("@opensumi/ide-editor/lib/browser");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const line_range_1 = require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/model/line-range");
const types_1 = require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/types");
const merge_editor_module_less_1 = tslib_1.__importDefault(require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/view/merge-editor.module.less"));
const stop_widget_1 = require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/widget/stop-widget");
const monaco_api_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api");
const languages_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api/languages");
const position_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/common/core/position");
const cache_conflicts_1 = require("./cache-conflicts");
const override_resolve_result_widget_1 = require("./override-resolve-result-widget");
var MERGE_CONFLICT;
(function (MERGE_CONFLICT) {
    const CATEGORY = 'MergeConflict';
    MERGE_CONFLICT.AI_ACCEPT = {
        id: 'merge-conflict.ai.accept',
        category: CATEGORY,
    };
    MERGE_CONFLICT.ALL_RESET = {
        id: 'merge-conflict.ai.all-reset',
        category: CATEGORY,
    };
    MERGE_CONFLICT.AI_ALL_ACCEPT = {
        id: 'merge-conflict.ai.all-accept',
        category: CATEGORY,
    };
    MERGE_CONFLICT.AI_ALL_ACCEPT_STOP = {
        id: 'merge-conflict.ai.all-accept-stop',
        category: CATEGORY,
    };
})(MERGE_CONFLICT = exports.MERGE_CONFLICT || (exports.MERGE_CONFLICT = {}));
const MERGE_CONFLICT_CODELENS_STYLE = 'merge-conflict-codelens-style';
// codelens 无法定义样式 此处通过 css hack
const cssStyle = `
.monaco-editor .codelens-decoration {
  & > a[title*=AI] {
    background-image: radial-gradient(circle at -21% -22%, #00f6ff, #9c03ff);
    color: transparent;
    background-clip: text;
  }
  & > a[title*=AI]:hover {
    background-image: radial-gradient(circle at -21% -22%, #00f6ff, #9c03ff);
    color: transparent !important;
    background-clip: text;
  }
  .codicon-ai-magic::before {
    font-family: 'kaitian-icon';
    content: "\\e60e";
  }
  .codicon-ai-magic {
    font: normal normal normal 16px/1 'kaitian-icon';
    display: inline-block;
    text-decoration: none;
    text-rendering: auto;
    text-align: center;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
    user-select: none;
    -webkit-user-select: none;
  }
  .codicon-modifier-spin {
    color: #00f6ff !important;
  }
}
`;
function loadStyleString(load) {
    let mergeConflictStyleNode = document.getElementById(MERGE_CONFLICT_CODELENS_STYLE);
    if (!load) {
        mergeConflictStyleNode === null || mergeConflictStyleNode === void 0 ? void 0 : mergeConflictStyleNode.remove();
        return;
    }
    if (!mergeConflictStyleNode && load) {
        mergeConflictStyleNode = document.createElement('style');
        mergeConflictStyleNode.id = MERGE_CONFLICT_CODELENS_STYLE;
        mergeConflictStyleNode.appendChild(document.createTextNode(cssStyle));
        document.getElementsByTagName('head')[0].appendChild(mergeConflictStyleNode);
    }
}
class WidgetFactory {
    constructor(contentWidget, editor, injector) {
        this.contentWidget = contentWidget;
        this.editor = editor;
        this.injector = injector;
        this.widgetMap = new Map();
    }
    hasWidget(range) {
        return this.widgetMap.get(range.id) !== undefined;
    }
    hideWidget(id) {
        if (id) {
            const widget = this.widgetMap.get(id);
            if (widget) {
                widget.hide();
                this.widgetMap.delete(id);
            }
            return;
        }
        this.widgetMap.forEach((widget) => {
            widget.hide();
        });
        this.widgetMap.clear();
    }
    addWidget(range) {
        const id = range.id;
        if (this.widgetMap.has(id)) {
            return;
        }
        const position = new position_1.Position(range.endLineNumberExclusive, 1);
        const widget = this.injector.get(this.contentWidget, [this.editor, range]);
        widget.show({ position });
        this.widgetMap.set(id, widget);
    }
}
let MergeConflictContribution = class MergeConflictContribution extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.cancelIndicatorMap = new Map();
        this.cacheResolvedConflicts = new Map();
        this._onRequestCancel = new ide_core_common_1.Emitter();
        this.onRequestCancel = this._onRequestCancel.event;
        this._onRequestsCancel = new ide_core_common_1.Emitter();
        this.onRequestsCancel = this._onRequestsCancel.event;
        // for codelens loading
        this.loadingRange = new Set();
        // for report
        this.currentReportMap = new Map();
        // for decoration dispose
        this.decorationId2Dispose = new Map();
        this.decorationId2Range = new Map();
        // 强制刷新 codelens
        this.updateCodeLensProvider = (0, debounce_1.default)(() => {
            if (this.getModel().uri) {
                // @ts-ignore
                languages_1.languageFeaturesService.codeLensProvider._onDidChange.fire();
            }
        }, 2000);
    }
    dispose() {
        super.dispose();
        this.cancelRequestToken();
        this.mergeConflictReportService.dispose();
        this.cacheConflicts.dispose();
    }
    onDidStart() {
        // 提供了 provider 才允许注册
        const handler = this.resolveConflictRegistry.getTraditionalHandler();
        if (this.aiNativeConfigService.capabilities.supportsConflictResolve && handler) {
            this.disposables.push(this.eventBus.on(ide_core_common_1.ExtensionActivatedEvent, (e) => {
                var _a;
                // 当插件注册完毕 再初始化 codelens 使其位置正确
                if (e.payload.topic === 'onExtensionActivated' &&
                    ((_a = e.payload.data) === null || _a === void 0 ? void 0 : _a.id).endsWith('.merge-conflict')) {
                    this.registerCodeLensProvider();
                    loadStyleString(true);
                }
            }), 
            // TODO 优化使用 registerEditorFeature
            this.editorService.onActiveResourceChange(() => {
                this.contribute(this.editorService.currentEditor);
            }));
        }
        else {
            loadStyleString(false);
        }
    }
    initListenEvent() {
        if (this.aiNativeConfigService.capabilities.supportsConflictResolve) {
            this.disposables.push(this.editor.onDidChangeModelContent(({ changes, eol }) => {
                const deltaEdits = [];
                changes.forEach((change) => {
                    var _a;
                    const { text, range } = change;
                    const textLineCount = ((_a = text.match(new RegExp(eol, 'ig'))) !== null && _a !== void 0 ? _a : []).length;
                    const { startLineNumber, endLineNumber } = range;
                    /**
                     * startLineNumber 与 endLineNumber 的差值表示选区选了多少行
                     * textLineCount 则表示文本出现的换行符数量
                     * 两者相加就得出此次文本变更最终新增或减少了多少行
                     */
                    const offset = startLineNumber - endLineNumber + textLineCount;
                    if (offset === 0) {
                        return;
                    }
                    deltaEdits.push({
                        startLineNumber,
                        endLineNumber,
                        offset,
                    });
                });
                deltaEdits.forEach((edits) => {
                    const map = new Map();
                    const { startLineNumber, endLineNumber, offset } = edits;
                    for (const [id, value] of this.getCacheResolvedConflicts().entries()) {
                        const { newRange } = value;
                        const { startLineNumber: newStartLineNumber, endLineNumber: newEndLineNumber, endColumn: newEndColumn, } = newRange;
                        // 在冲突位置下方改动 不处理
                        if (newEndLineNumber < startLineNumber) {
                            map.set(id, value);
                            continue;
                        }
                        // 在冲突位置上方改动 偏移
                        if (startLineNumber < newStartLineNumber && endLineNumber < newEndLineNumber) {
                            const newRange = new monaco.Range(newStartLineNumber + offset, 1, newEndLineNumber + offset, newEndColumn);
                            map.set(id, Object.assign(Object.assign({}, value), { newRange }));
                            continue;
                        }
                        // 包含冲突位置 删除
                        if (startLineNumber <= newStartLineNumber && endLineNumber >= newEndLineNumber) {
                            const newRangeOffset = newEndLineNumber - newStartLineNumber;
                            if (offset <= newRangeOffset) {
                                continue;
                            }
                        }
                        // 内部改动
                        if (startLineNumber >= newStartLineNumber && endLineNumber <= newEndLineNumber) {
                            // 内部改动删除
                            // TODO
                            // const newRange = new monaco.Range(newStartLineNumber, 1, newEndLineNumber + offset, 1);
                            // map.set(id, {
                            //   ...value,
                            //   newRange,
                            // });
                        }
                    }
                    this.resetCacheResolvedConflicts(map);
                    for (const [id, value] of this.decorationId2Range.entries()) {
                        const { startLineNumber: decorationStartLineNumber, endLineNumber: decorationEndLineNumber, endColumn: decorationEndColumn, } = value;
                        // 在位置下方改动 不处理
                        if (decorationEndLineNumber < startLineNumber) {
                            continue;
                        }
                        // 在位置上方改动 偏移
                        if (startLineNumber < decorationStartLineNumber && endLineNumber < decorationEndLineNumber) {
                            const newRange = new monaco.Range(decorationStartLineNumber + offset, 1, decorationEndLineNumber + offset, decorationEndColumn);
                            this.decorationId2Range.set(id, newRange);
                            continue;
                        }
                        // 包含位置 删除
                        if (startLineNumber <= decorationStartLineNumber && endLineNumber >= decorationEndLineNumber) {
                            const newRangeOffset = decorationEndLineNumber - decorationStartLineNumber;
                            if (offset <= newRangeOffset) {
                                this.cleanDecoration(id);
                                continue;
                            }
                        }
                        // 内部改动删除
                        if (startLineNumber >= decorationStartLineNumber && endLineNumber <= decorationEndLineNumber) {
                            this.cleanDecoration(id);
                        }
                    }
                });
                this.updateAllWidgets();
            }), this.editor.onDidChangeModel(() => {
                this.updateAllWidgets();
            }));
        }
    }
    init() {
        this.initListenEvent();
    }
    /* report */
    get reportData() {
        const uri = this.getUri();
        const reportData = this.currentReportMap.get(uri);
        if (reportData) {
            return reportData;
        }
        else {
            const reportInfo = {
                editorMode: ide_core_common_1.MergeConflictEditorMode.traditional,
                conflictPointNum: 0,
                useAiConflictPointNum: 0,
                receiveNum: 0,
                clickNum: 0,
                clickAllNum: 0,
                aiOutputNum: 0,
                cancelNum: 0,
            };
            this.currentReportMap.set(uri, reportInfo);
            this.mergeConflictReportService.report(uri, reportInfo);
            return reportInfo;
        }
    }
    set reportData(data) {
        const uri = this.getUri();
        const reportData = this.reportData;
        this.currentReportMap.set(uri, Object.assign(Object.assign({}, reportData), data));
    }
    reportConflictData() {
        const uri = this.getUri();
        const reportData = this.currentReportMap.get(uri);
        if (reportData) {
            this.mergeConflictReportService.report(this.getUri(), this.reportData);
        }
    }
    updateReportData() {
        var _a;
        const allConflictCache = this.cacheConflicts.getAllConflictsByUri(this.getUri());
        let conflictPointNum = 0;
        let useAiConflictPointNum = 0;
        let receiveNum = 0;
        conflictPointNum = (allConflictCache === null || allConflictCache === void 0 ? void 0 : allConflictCache.length) || 0;
        allConflictCache === null || allConflictCache === void 0 ? void 0 : allConflictCache.forEach((cacheConflict) => {
            if (cacheConflict.isResolved) {
                useAiConflictPointNum += 1;
            }
        });
        // 内部修改 删除态无法统计
        for (const [uri, cacheResolvedConflictsMap] of this.cacheResolvedConflicts.entries()) {
            for (const [, cacheResolvedConflicts] of cacheResolvedConflictsMap.entries()) {
                // 统计当前文件
                if (uri === this.getModel().uri.toString()) {
                    // 计算当前文件采纳数量
                    const aiResult = cacheResolvedConflicts.textChange.newText;
                    const currentResult = (_a = this.getModel()) === null || _a === void 0 ? void 0 : _a.getValueInRange(cacheResolvedConflicts.newRange);
                    if (aiResult.trim() === currentResult.trim()) {
                        cacheResolvedConflicts.isAccept = true;
                        receiveNum += 1;
                    }
                    else {
                        cacheResolvedConflicts.isAccept = false;
                    }
                }
            }
        }
        this.reportData = {
            conflictPointNum,
            useAiConflictPointNum,
            receiveNum,
        };
    }
    /* report */
    getAllDiffRanges() {
        const rangeLines = [];
        for (const [, value] of this.getCacheResolvedConflicts().entries()) {
            rangeLines.push(value.newRange);
        }
        return rangeLines;
    }
    updateAllWidgets() {
        this.hideResolveResultWidget();
        for (const [id, value] of this.getCacheResolvedConflicts().entries()) {
            if (!value.isClosed) {
                const lineRange = this.toLineRange(value.newRange, id);
                this.resolveResultWidgetManager.addWidget(lineRange);
            }
        }
    }
    getModel() {
        return this.editor.getModel();
    }
    getUri() {
        return this.getModel().uri.toString();
    }
    /* cache widget */
    getCacheResolvedConflicts(currentUri) {
        if (!currentUri) {
            currentUri = this.getModel().uri.toString();
        }
        const cache = this.cacheResolvedConflicts.get(currentUri);
        if (cache) {
            return cache;
        }
        else {
            this.cacheResolvedConflicts.set(currentUri, new Map());
            return this.cacheResolvedConflicts.get(currentUri);
        }
    }
    deleteCacheResolvedConflicts(id, currentUri) {
        if (!currentUri) {
            currentUri = this.getModel().uri.toString();
        }
        const cache = this.cacheResolvedConflicts.get(currentUri);
        if (cache) {
            cache.delete(id);
        }
    }
    setCacheResolvedConflict(id, cacheConflict, currentUri) {
        if (!currentUri) {
            currentUri = this.getModel().uri.toString();
        }
        const cache = this.getCacheResolvedConflicts(currentUri);
        cache.set(id, cacheConflict);
    }
    resetCacheResolvedConflicts(cacheConflicts, currentUri) {
        if (!currentUri) {
            currentUri = this.getModel().uri.toString();
        }
        if (this.cacheResolvedConflicts.has(currentUri)) {
            this.cacheResolvedConflicts.set(currentUri, cacheConflicts);
        }
    }
    toLineRange(range, id) {
        const lineRange = new line_range_1.LineRange(range.startLineNumber, range.endLineNumber);
        if (id) {
            // @ts-ignore
            lineRange.setId(id);
        }
        return lineRange;
    }
    renderSkeletonDecoration(range, classNames) {
        const model = this.getModel();
        const renderSkeletonDecoration = (className) => ({
            range,
            options: {
                isWholeLine: true,
                description: 'skeleton',
                className,
            },
        });
        const preDecorationsIds = (model === null || model === void 0 ? void 0 : model.deltaDecorations([], classNames.map((cls) => renderSkeletonDecoration(cls)))) || [];
        return () => {
            model.deltaDecorations(preDecorationsIds, []);
        };
    }
    registerCommands(commands) {
        this.disposables.push(commands.registerCommand(MERGE_CONFLICT.AI_ACCEPT, {
            execute: async (type, conflict) => {
                this.conflictAIAccept(conflict);
            },
        }), commands.registerCommand(MERGE_CONFLICT.ALL_RESET, {
            execute: async (uri) => {
                var _a, _b, _c;
                const content = this.cacheConflicts.getConflictText(uri.toString());
                this.cancelRequestToken();
                if (content) {
                    if (((_b = (_a = this.editorService.currentEditor) === null || _a === void 0 ? void 0 : _a.currentUri) === null || _b === void 0 ? void 0 : _b.toString()) === uri.toString()) {
                        const editor = (_c = this.editorService.currentEditor) === null || _c === void 0 ? void 0 : _c.monacoEditor;
                        if (editor) {
                            const model = editor.getModel();
                            model === null || model === void 0 ? void 0 : model.setValue(content);
                            this.cacheConflicts.deleteConflictText(uri.toString());
                            this.cleanAllCache();
                        }
                    }
                }
            },
        }), commands.registerCommand(MERGE_CONFLICT.AI_ALL_ACCEPT, {
            execute: async () => {
                const document = this.getModel();
                if (!document) {
                    return Promise.resolve();
                }
                const conflicts = this.cacheConflicts.scanDocument(document);
                if (!(conflicts === null || conflicts === void 0 ? void 0 : conflicts.length)) {
                    return Promise.resolve();
                }
                this.reportData = {
                    clickAllNum: this.reportData.clickAllNum + 1,
                };
                //  一个个解决
                await this.acceptAllConflict();
            },
        }), commands.registerCommand(MERGE_CONFLICT.AI_ALL_ACCEPT_STOP, {
            execute: async () => {
                this.cancelRequestToken();
            },
        }), commands.afterExecuteCommand('git.stage', (args) => {
            this.reportConflictData();
            return args;
        }), commands.afterExecuteCommand('git.stageAllMerge', (args) => {
            this.reportConflictData();
            return args;
        }));
    }
    contribute(editor) {
        if (!editor) {
            return this;
        }
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return this;
        }
        const currentEditor = editor;
        if (currentEditor && monacoEditor && !this.editor) {
            this.editor = monacoEditor;
            this.resolveResultWidgetManager = new WidgetFactory(override_resolve_result_widget_1.OverrideResolveResultWidget, this, this.injector);
            this.stopWidgetManager = new WidgetFactory(stop_widget_1.StopWidget, this, this.injector);
            this.init();
        }
    }
    async provideCodeLens(document, _token) {
        const conflicts = this.cacheConflicts.scanDocument(document);
        if (!conflicts.length) {
            return null;
        }
        const items = [];
        conflicts.forEach((conflict) => {
            const aiAcceptCommand = {
                id: MERGE_CONFLICT.AI_ACCEPT.id,
                title: `$(ai-magic) ${(0, ide_core_common_1.localize)('mergeEditor.conflict.resolve.all')}`,
                arguments: ['know-conflict', conflict],
                tooltip: (0, ide_core_common_1.localize)('mergeEditor.conflict.resolve.all'),
            };
            // loading 效果
            this.loadingRange.forEach((range) => {
                if (conflict.range.equalsRange(range)) {
                    aiAcceptCommand.title = `$(loading~spin) ${(0, ide_core_common_1.localize)('mergeEditor.conflict.resolve.all')}`;
                }
            });
            items.push({
                range: conflict.range,
                command: aiAcceptCommand,
                id: MERGE_CONFLICT.AI_ACCEPT.id,
            });
        });
        return Promise.resolve(items);
    }
    async conflictAIAccept(conflict, lineRan, isRegenerate) {
        var _a, _b, _c, _d, _e, _f;
        if (!((_a = this.editorService.currentEditor) === null || _a === void 0 ? void 0 : _a.monacoEditor)) {
            return;
        }
        let lineRange;
        let conflictMetadata;
        const model = this.getModel();
        if (conflict) {
            lineRange = this.toLineRange(conflict.range);
            conflictMetadata = {
                base: '',
                current: model.getValueInRange(conflict.current.content),
                incoming: model.getValueInRange(conflict.incoming.content),
                currentName: conflict.current.name,
                incomingName: conflict.incoming.name,
            };
        }
        else {
            lineRange = lineRan;
        }
        const range = lineRange.toRange(1, (_b = conflict === null || conflict === void 0 ? void 0 : conflict.range.endColumn) !== null && _b !== void 0 ? _b : 1073741824 /* Constants.MAX_SAFE_SMALL_INTEGER */);
        const skeletonDecorationDispose = this.renderSkeletonDecoration(range, [
            merge_editor_module_less_1.default.skeleton_decoration,
            merge_editor_module_less_1.default.skeleton_decoration_background_black,
        ]);
        this.stopWidgetManager.addWidget(lineRange);
        let codeAssemble = (_d = (_c = this.getModel()) === null || _c === void 0 ? void 0 : _c.getValueInRange(lineRange.toRange(1, range.endColumn))) !== null && _d !== void 0 ? _d : '';
        if (isRegenerate) {
            const cache = this.getCacheResolvedConflicts().get(lineRange.id);
            codeAssemble = (_e = cache === null || cache === void 0 ? void 0 : cache.conflictText) !== null && _e !== void 0 ? _e : '';
            conflictMetadata = cache === null || cache === void 0 ? void 0 : cache.metadata;
        }
        let resolveConflictResult;
        try {
            this.loadingRange.add(range);
            this.reportData = {
                clickNum: this.reportData.clickNum + 1,
            };
            resolveConflictResult = await this.requestAiResolveConflict(conflictMetadata, lineRange, isRegenerate);
        }
        catch (error) {
            throw new Error(`AI resolve conflict error: ${error.toString()}`);
        }
        finally {
            skeletonDecorationDispose();
            this.stopWidgetManager.hideWidget(lineRange.id);
            this.loadingRange.delete(range);
            this.updateCodeLensProvider();
        }
        if (ide_core_common_1.ReplyResponse.is(resolveConflictResult)) {
            this.reportData = {
                aiOutputNum: this.reportData.aiOutputNum + 1,
            };
            const { text, lineNumber, lines } = this.resolveEndLineEOL(resolveConflictResult.message);
            const endLineNumber = lineRange.startLineNumber + lineNumber - 1;
            const endColumn = lines[lines.length - 1].length + 1;
            const newRange = new monaco.Range(lineRange.startLineNumber, 1, endLineNumber, endColumn);
            const edit = {
                range,
                text,
            };
            let validEditOperation = [];
            const selections = (_f = this.getModel()) === null || _f === void 0 ? void 0 : _f.pushEditOperations(null, [edit], (operation) => {
                validEditOperation = operation;
                const selections = [];
                operation.forEach((op) => {
                    selections.push(new monaco.Selection(op.range.startLineNumber, op.range.startColumn, op.range.endLineNumber, op.range.endColumn));
                });
                return selections;
            });
            selections.forEach((selection) => {
                const decorationDispose = this.renderSkeletonDecoration(selection, [merge_editor_module_less_1.default.skeleton_decoration_complete]);
                this.decorationId2Dispose.set(lineRange.id, decorationDispose);
                this.decorationId2Range.set(lineRange.id, selection);
            });
            const newLineRange = this.toLineRange(newRange, lineRange.id);
            this.resolveResultWidgetManager.addWidget(newLineRange);
            this.setCacheResolvedConflict(newLineRange.id, Object.assign(Object.assign({}, validEditOperation[0]), { newRange, id: newLineRange.id, metadata: conflictMetadata, 
                // 保留原始冲突文本
                conflictText: (isRegenerate && codeAssemble) || validEditOperation[0].text, isAccept: true }));
            if (!isRegenerate) {
                // 记录处理数量 非重新生成 conflict 存在
                const uri = this.getModel().uri.toString();
                const cacheConflictRanges = this.cacheConflicts.getAllConflictsByUri(uri);
                if (cacheConflictRanges) {
                    const cacheConflict = cacheConflictRanges.find((cacheConflict) => {
                        if (cacheConflict.isResolved) {
                            return false;
                        }
                        if (cacheConflict.range.equalsRange(range)) {
                            return true;
                        }
                        if (cacheConflict.text === codeAssemble) {
                            return true;
                        }
                    });
                    if (cacheConflict && !cacheConflict.isResolved) {
                        this.cacheConflicts.setConflictResolved(uri, cacheConflict.id);
                    }
                }
            }
            this.updateReportData();
            this.reportConflictData();
            return Promise.resolve(resolveConflictResult);
        }
        else {
            if (ide_core_common_1.CancelResponse.is(resolveConflictResult)) {
                this.reportData = {
                    cancelNum: this.reportData.cancelNum + 1,
                };
                this.reportConflictData();
            }
            if (ide_core_common_1.ErrorResponse.is(resolveConflictResult)) {
                this.loadingRange.delete(range);
                return Promise.resolve(resolveConflictResult);
            }
        }
    }
    resolveEndLineEOL(text) {
        var _a, _b;
        const eol = (_b = (_a = this.getModel()) === null || _a === void 0 ? void 0 : _a.getEOL()) !== null && _b !== void 0 ? _b : '\n';
        const lines = text.split(eol);
        const lastLine = lines[lines.length - 1];
        // 最后一行去掉换行符
        if (lastLine === '' && lines.length > 1) {
            lines.pop();
        }
        return {
            text: lines.join(eol),
            lineNumber: lines.length,
            lines,
        };
    }
    async acceptAllConflict() {
        const document = this.getModel();
        if (!document) {
            return Promise.resolve();
        }
        const conflicts = this.cacheConflicts.scanDocument(document);
        if (!conflicts.length) {
            return Promise.resolve();
        }
        let isCancelAll = false;
        // TODO 优化
        this.disposables.push(this.onRequestsCancel(() => {
            isCancelAll = true;
            return;
        }), 
        // 取消一个后取消所有
        this.onRequestCancel(() => {
            isCancelAll = true;
            return;
        }));
        return this.conflictAIAccept(conflicts[0]).then((res) => {
            if (isCancelAll) {
                return Promise.resolve();
            }
            if (!ide_core_common_1.ErrorResponse.is(res)) {
                return Promise.resolve();
            }
            return this.acceptAllConflict();
        });
    }
    registerCodeLensProvider() {
        return monaco_api_1.monacoApi.languages.registerCodeLensProvider([{ scheme: 'file' }], {
            provideCodeLenses: async (model, token) => {
                let lens = await this.provideCodeLens(model, token);
                return {
                    lenses: lens !== null && lens !== void 0 ? lens : [],
                    dispose: () => {
                        lens = [];
                    },
                };
            },
        });
    }
    async requestAiResolveConflict(metadata, range, isRegenerate = false) {
        const handler = this.resolveConflictRegistry.getTraditionalHandler();
        if (!handler) {
            return;
        }
        if (isRegenerate) {
            const newContent = this.getModel().getValueInRange(range.toRange());
            metadata.resultContent = newContent;
        }
        const response = await handler.providerRequest(metadata, { isRegenerate }, this.createRequestToken(range.id).token);
        return response;
    }
    // 生成 cancel token
    createRequestToken(id) {
        const token = new ide_core_common_1.CancellationTokenSource();
        this.cancelIndicatorMap.set(id, token);
        return token;
    }
    hideResolveResultWidget(id) {
        this.resolveResultWidgetManager.hideWidget(id);
    }
    hideStopWidget(id) {
        this.stopWidgetManager.hideWidget(id);
    }
    cancelRequestToken(id) {
        if (id) {
            if (!this.cancelIndicatorMap.has(id)) {
                return;
            }
            const token = this.cancelIndicatorMap.get(id);
            token === null || token === void 0 ? void 0 : token.cancel();
            this._onRequestCancel.fire({ id, type: 'cancel' });
            return;
        }
        const requestsCancel = [];
        this.cancelIndicatorMap.forEach((token, id) => {
            token.cancel();
            requestsCancel.push({ id, type: 'cancel' });
        });
        this._onRequestsCancel.fire(requestsCancel);
        this.cancelIndicatorMap.clear();
    }
    launchConflictActionsEvent(eventData) {
        var _a;
        const { range, action } = eventData;
        if (action === types_1.REVOKE_ACTIONS) {
            const cacheConflict = this.getCacheResolvedConflicts().get(range.id);
            if (cacheConflict) {
                const edit = {
                    range: cacheConflict.newRange,
                    text: cacheConflict.conflictText || cacheConflict.text,
                };
                (_a = this.getModel()) === null || _a === void 0 ? void 0 : _a.pushEditOperations(null, [edit], () => null);
            }
            this.cleanWidget(range.id);
            this.deleteCacheResolvedConflicts(range.id);
            this.cleanDecoration(range.id);
        }
        else if (action === types_1.AI_RESOLVE_REGENERATE_ACTIONS) {
            this.cleanWidget(range.id);
            this.conflictAIAccept(undefined, range, true);
            this.cleanDecoration(range.id);
        }
        else if (action === types_1.IGNORE_ACTIONS) {
            this.cleanWidget(range.id);
            // this.deleteCacheResolvedConflicts(range.id);
            const resolvedConflict = this.getCacheResolvedConflicts().get(range.id);
            this.setCacheResolvedConflict(range.id, Object.assign(Object.assign({}, resolvedConflict), { isClosed: true }));
        }
    }
    cleanAllCache() {
        this.hideStopWidget();
        this.hideResolveResultWidget();
        this.getCacheResolvedConflicts().clear();
        this.decorationId2Dispose.clear();
        this.decorationId2Range.clear();
    }
    cleanWidget(id) {
        if (id) {
            this.hideStopWidget(id);
            this.hideResolveResultWidget(id);
            return;
        }
        this.hideStopWidget();
        this.hideResolveResultWidget();
    }
    cleanDecoration(id) {
        var _a;
        if (this.decorationId2Dispose.has(id)) {
            (_a = this.decorationId2Dispose.get(id)) === null || _a === void 0 ? void 0 : _a.call(this);
            this.decorationId2Dispose.delete(id);
        }
        if (this.decorationId2Range.has(id)) {
            this.decorationId2Range.delete(id);
        }
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], MergeConflictContribution.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(browser_1.WorkbenchEditorService),
    tslib_1.__metadata("design:type", browser_1.WorkbenchEditorService)
], MergeConflictContribution.prototype, "editorService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IEventBus),
    tslib_1.__metadata("design:type", Object)
], MergeConflictContribution.prototype, "eventBus", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AINativeConfigService),
    tslib_1.__metadata("design:type", ide_core_browser_1.AINativeConfigService)
], MergeConflictContribution.prototype, "aiNativeConfigService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", cache_conflicts_1.CacheConflict)
], MergeConflictContribution.prototype, "cacheConflicts", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.AIBackSerivcePath),
    tslib_1.__metadata("design:type", Object)
], MergeConflictContribution.prototype, "aiBackService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(conflict_report_service_1.MergeConflictReportService),
    tslib_1.__metadata("design:type", conflict_report_service_1.MergeConflictReportService)
], MergeConflictContribution.prototype, "mergeConflictReportService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ResolveConflictRegistryToken),
    tslib_1.__metadata("design:type", Object)
], MergeConflictContribution.prototype, "resolveConflictRegistry", void 0);
MergeConflictContribution = tslib_1.__decorate([
    (0, ide_core_common_1.Domain)(ide_core_common_1.CommandContribution, ide_core_browser_1.ClientAppContribution)
], MergeConflictContribution);
exports.MergeConflictContribution = MergeConflictContribution;
//# sourceMappingURL=index.js.map