"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MsgHistoryManager = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
let MsgHistoryManager = class MsgHistoryManager extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.messageMap = new Map();
        this._onMessageChange = new ide_core_common_1.Emitter();
        this.onMessageChange = this._onMessageChange.event;
    }
    doAddMessage(message) {
        const id = (0, ide_core_common_1.uuid)(6);
        const order = this.messageMap.size;
        const msg = Object.assign(Object.assign({}, message), { id,
            order, type: message.type || 'string' });
        this.messageMap.set(id, msg);
        this._onMessageChange.fire(this.getMessages());
        return id;
    }
    getMessages() {
        return Array.from(this.messageMap.values()).sort((a, b) => a.order - b.order);
    }
    addUserMessage(message) {
        return this.doAddMessage(Object.assign(Object.assign({}, message), { role: 1 /* ChatMessageRole.User */ }));
    }
    addAgentMessage(message) {
        return this.doAddMessage(Object.assign(Object.assign({}, message), { role: 1 /* ChatMessageRole.User */ }));
    }
    addAssistantMessage(message) {
        return this.doAddMessage(Object.assign(Object.assign({}, message), { role: 2 /* ChatMessageRole.Assistant */ }));
    }
    updateAssistantMessage(id, message) {
        if (!this.messageMap.has(id)) {
            return;
        }
        const oldMessage = this.messageMap.get(id);
        this.messageMap.set(id, Object.assign(Object.assign({}, oldMessage), { content: message.content }));
    }
};
MsgHistoryManager = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: false })
], MsgHistoryManager);
exports.MsgHistoryManager = MsgHistoryManager;
//# sourceMappingURL=msg-history-manager.js.map