"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MsgStreamManager = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
let MsgStreamManager = class MsgStreamManager extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        // 会话 id 与接口 answer id 的对应关系
        this.sessionIdToAnswerIdMap = new Map();
        // 每个 answer id 所对应的消息流
        this.answerIdToMsgStreamMap = new Map();
        this.onDidMsgListChangeDispatcher = this.registerDispose(new ide_core_common_1.Dispatcher());
        this._onMsgStatus = this.registerDispose(new ide_core_common_1.Emitter());
        this.onMsgStatus = this._onMsgStatus.event;
    }
    get status() {
        return this._status;
    }
    set status(s) {
        this._status = s;
        this._onMsgStatus.fire(s);
    }
    get currentSessionId() {
        return this._currentSessionId;
    }
    setCurrentSessionId(id) {
        this._currentSessionId = id;
        this.status = 0 /* EMsgStreamStatus.READY */;
    }
    onMsgListChange(sessionId) {
        return this.onDidMsgListChangeDispatcher.on(sessionId);
    }
    sendErrorStatue() {
        this.status = 3 /* EMsgStreamStatus.ERROR */;
    }
    sendDoneStatue() {
        this.status = 2 /* EMsgStreamStatus.DONE */;
    }
    sendThinkingStatue() {
        this.status = 1 /* EMsgStreamStatus.THINKING */;
    }
    sendPauseStatue() {
        this.status = 4 /* EMsgStreamStatus.PAUSE */;
    }
    recordMessage(answerId, msg) {
        if (!this._currentSessionId) {
            new Error('currentSessionId is null');
        }
        this.sessionIdToAnswerIdMap.set(this.currentSessionId, answerId);
        if (!(answerId && msg)) {
            new Error('answerId/msg is null');
        }
        const answerList = this.answerIdToMsgStreamMap.get(answerId);
        if (!answerList) {
            this.answerIdToMsgStreamMap.set(answerId, [msg]);
        }
        else {
            answerList.push(msg);
        }
        const { finish_reason } = msg;
        if (!finish_reason) {
            this.sendThinkingStatue();
        }
        else if (finish_reason === 'stop') {
            this.sendDoneStatue();
        }
        else {
            this.sendErrorStatue();
            return;
        }
        this.onDidMsgListChangeDispatcher.dispatch(this._currentSessionId, msg);
    }
};
MsgStreamManager = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: false })
], MsgStreamManager);
exports.MsgStreamManager = MsgStreamManager;
//# sourceMappingURL=msg-stream-manager.js.map