"use strict";
var InlineChatFeatureRegistry_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.InlineChatFeatureRegistry = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
let InlineChatFeatureRegistry = InlineChatFeatureRegistry_1 = class InlineChatFeatureRegistry extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.actionsMap = new Map();
        this.codeActionsMap = new Map();
        this.editorHandlerMap = new Map();
        this.terminalHandlerMap = new Map();
        this._onCodeActionRun = new ide_core_common_1.Emitter();
        this.onCodeActionRun = this._onCodeActionRun.event;
    }
    dispose() {
        super.dispose();
        this.actionsMap.clear();
        this.editorHandlerMap.clear();
    }
    static getCommandId(type, id) {
        return `ai-native.inline-chat.${type}.${id}`;
    }
    collectActions(type, operational) {
        var _a;
        const { id } = operational;
        if (this.actionsMap.has(id)) {
            this.logger.warn(`InlineChatFeatureRegistry: id ${id} already exists`);
            return false;
        }
        this.disposables.push(this.commandRegistry.registerCommand({
            id: InlineChatFeatureRegistry_1.getCommandId(type, id),
        }, {
            execute: async (range) => {
                this._onCodeActionRun.fire({
                    id,
                    range,
                });
            },
        }));
        this.actionsMap.set(id, operational);
        if (operational.codeAction) {
            const { codeAction } = operational;
            const action = {
                title: codeAction.title || operational.name,
                isAI: true,
                isPreferred: (_a = codeAction.isPreferred) !== null && _a !== void 0 ? _a : true,
                kind: codeAction.kind || 'InlineChat',
                disabled: codeAction.disabled,
                command: {
                    id: InlineChatFeatureRegistry_1.getCommandId('editor', operational.id),
                },
            };
            this.codeActionsMap.set(id, action);
        }
        return true;
    }
    removeCollectedActions(type, operational) {
        this.actionsMap.delete(operational.id);
        this.codeActionsMap.delete(operational.id);
        this.commandRegistry.unregisterCommand(InlineChatFeatureRegistry_1.getCommandId(type, operational.id));
    }
    registerEditorInlineChat(operational, handler) {
        const isCollect = this.collectActions('editor', operational);
        if (isCollect) {
            this.editorHandlerMap.set(operational.id, handler);
        }
        return {
            dispose: () => {
                this.removeCollectedActions('editor', operational);
            },
        };
    }
    registerTerminalInlineChat(operational, handler) {
        const isCollect = this.collectActions('terminal', operational);
        if (isCollect) {
            if ((0, ide_core_common_1.isUndefined)(handler.triggerRules)) {
                handler.triggerRules = 'selection';
            }
            this.terminalHandlerMap.set(operational.id, handler);
        }
        return {
            dispose: () => {
                this.removeCollectedActions('terminal', operational);
            },
        };
    }
    getEditorActionButtons() {
        return Array.from(this.editorHandlerMap.keys())
            .filter((id) => {
            const actions = this.actionsMap.get(id);
            return actions && actions.renderType === 'button';
        })
            .map((id) => this.actionsMap.get(id));
    }
    getCodeActions() {
        return Array.from(this.codeActionsMap.values());
    }
    getEditorActionMenus() {
        return Array.from(this.editorHandlerMap.keys())
            .filter((id) => {
            const actions = this.actionsMap.get(id);
            return actions && actions.renderType === 'dropdown';
        })
            .map((id) => this.actionsMap.get(id));
    }
    getEditorHandler(id) {
        return this.editorHandlerMap.get(id);
    }
    getTerminalHandler(id) {
        return this.terminalHandlerMap.get(id);
    }
    getTerminalActions() {
        return Array.from(this.terminalHandlerMap.keys()).map((id) => this.actionsMap.get(id));
    }
    getAction(id) {
        return this.actionsMap.get(id);
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.Logger),
    tslib_1.__metadata("design:type", ide_core_browser_1.Logger)
], InlineChatFeatureRegistry.prototype, "logger", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.CommandRegistry),
    tslib_1.__metadata("design:type", Object)
], InlineChatFeatureRegistry.prototype, "commandRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.CommandService),
    tslib_1.__metadata("design:type", Object)
], InlineChatFeatureRegistry.prototype, "commandService", void 0);
InlineChatFeatureRegistry = InlineChatFeatureRegistry_1 = tslib_1.__decorate([
    (0, di_1.Injectable)()
], InlineChatFeatureRegistry);
exports.InlineChatFeatureRegistry = InlineChatFeatureRegistry;
//# sourceMappingURL=inline-chat.feature.registry.js.map