"use strict";
var AIDiffWidget_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIDiffWidget = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const client_1 = tslib_1.__importDefault(require("react-dom/client"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const monaco_api_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api");
const model_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/common/services/model");
const zoneWidget_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/contrib/zoneWidget/browser/zoneWidget");
const standaloneServices_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/standalone/browser/standaloneServices");
const inline_diff_widget_module_less_1 = tslib_1.__importDefault(require("./inline-diff-widget.module.less"));
const diffEditorOptions = {
    fixedOverflowWidgets: true,
    readOnly: true,
    enableSplitViewResizing: true,
    ignoreTrimWhitespace: false,
    renderSideBySide: true,
    lineNumbers: 'on',
    glyphMargin: true,
    scrollbar: { useShadows: false, alwaysConsumeMouseWheel: false, vertical: 'hidden' },
    scrollBeyondLastLine: false,
    renderMarginRevertIcon: false,
    renderOverviewRuler: false,
    rulers: undefined,
    overviewRulerBorder: undefined,
    overviewRulerLanes: 0,
    padding: { top: 0, bottom: 0 },
    folding: false,
    diffCodeLens: false,
    stickyScroll: { enabled: false },
    minimap: { enabled: false },
    renderLineHighlight: 'all',
    automaticLayout: true,
};
const DiffContentProvider = react_1.default.memo((props) => {
    const { dto, onMaxLincCount, editor } = props;
    const monacoService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.MonacoService);
    const editorRef = (0, react_1.useRef)(null);
    (0, react_1.useEffect)(() => {
        if (!dto) {
            return;
        }
        const model = editor.getModel();
        if (!model) {
            return;
        }
        const { selection, modifiedValue } = dto;
        const codeValueInRange = model.getValueInRange(selection);
        const diffEditor = monacoService.createDiffEditor(editorRef.current, Object.assign(Object.assign({}, diffEditorOptions), { lineDecorationsWidth: editor.getLayoutInfo().decorationsWidth, lineNumbersMinChars: editor.getOption(monaco.editor.EditorOption.lineNumbersMinChars) }));
        const modelService = standaloneServices_1.StandaloneServices.get(model_1.IModelService);
        const languageSelection = { languageId: model.getLanguageId(), onDidChange: ide_core_browser_1.Event.None };
        const originalModel = modelService.createModel(codeValueInRange, languageSelection);
        const modifiedModel = modelService.createModel(modifiedValue, languageSelection);
        diffEditor.setModel({
            original: originalModel,
            modified: modifiedModel,
        });
        diffEditor.revealLine(selection.startLineNumber, monaco.editor.ScrollType.Immediate);
        if (onMaxLincCount) {
            const originalEditor = diffEditor.getOriginalEditor();
            const modifiedEditor = diffEditor.getModifiedEditor();
            const originContentHeight = originalEditor.getContentHeight();
            const originLineCount = originContentHeight / originalEditor.getOption(monaco_api_1.monaco.editor.EditorOption.lineHeight);
            const modifiedContentHeight = modifiedEditor.getContentHeight();
            const modifiedLineCount = modifiedContentHeight / modifiedEditor.getOption(monaco_api_1.monaco.editor.EditorOption.lineHeight);
            onMaxLincCount(Math.max(originLineCount, modifiedLineCount) + 1);
        }
        return () => {
            if (diffEditor) {
                diffEditor.dispose();
            }
        };
    }, [dto]);
    return react_1.default.createElement("div", { ref: editorRef, className: inline_diff_widget_module_less_1.default.diff_editor_widget });
});
let AIDiffWidget = AIDiffWidget_1 = class AIDiffWidget extends zoneWidget_1.ZoneWidget {
    applyClass() { }
    applyStyle() { }
    _fillContainer(container) {
        this.setCssClass('ai_diff-widget');
        this.root = client_1.default.createRoot(container);
        this.root.render(react_1.default.createElement(ide_core_browser_1.ConfigProvider, { value: this.configContext },
            react_1.default.createElement("div", { className: inline_diff_widget_module_less_1.default.ai_diff_editor_container },
                react_1.default.createElement(DiffContentProvider, { dto: { selection: this.selection, modifiedValue: this.modifiedValue }, editor: this.editor, onMaxLincCount: (n) => {
                        if (n) {
                            this._relayout(n);
                            this._onMaxLincCount.fire(n);
                        }
                    } }))));
    }
    constructor(editor, selection, modifiedValue) {
        super(editor, {
            showArrow: false,
            showFrame: false,
            arrowColor: undefined,
            frameColor: undefined,
            keepEditorSelection: true,
        });
        this._onMaxLincCount = new ide_core_browser_1.Emitter();
        this.onMaxLincCount = this._onMaxLincCount.event;
        this.selection = selection;
        this.modifiedValue = modifiedValue;
    }
    // // 覆写 revealRange 函数，使其在 show 的时候编辑器不会定位到对应位置
    revealRange(range, isLastLine) {
        // not implement
    }
    setContainerStyle(style) {
        var _a;
        const keys = Object.keys(style);
        for (const key of keys) {
            if (Object.prototype.hasOwnProperty.call((_a = this.container) === null || _a === void 0 ? void 0 : _a.style, key)) {
                this.container.style[key] = style[key];
            }
        }
    }
    addClassName(type) {
        this.setCssClass(type);
        return this;
    }
    dispose() {
        this.hide();
        super.dispose();
    }
    hide() {
        this.editor.setHiddenAreas([], AIDiffWidget_1._hideId);
        super.hide();
        if (this.root) {
            this.root.unmount();
        }
    }
    showByLine(line, lineNumber = 20) {
        /**
         * 由于 monaco 在最新的版本中支持了 showInHiddenAreas 选项（见：https://github.com/microsoft/vscode/pull/181029），具备了在空白行显示 zonewidget 的能力
         * 所以这里暂时通过 hack 的方式使其能让 zonewidget 在空白处显示出来，后续需要升级 monaco 来实现
         */
        // @ts-ignore
        this.editor._modelData.viewModel.coordinatesConverter.modelPositionIsVisible = () => true;
        super.show({
            startLineNumber: line,
            startColumn: 1,
            endLineNumber: line,
            endColumn: Number.MAX_SAFE_INTEGER,
        }, lineNumber);
    }
};
AIDiffWidget._hideId = 'overlayDiff';
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AppConfig),
    tslib_1.__metadata("design:type", Object)
], AIDiffWidget.prototype, "configContext", void 0);
AIDiffWidget = AIDiffWidget_1 = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [Object, monaco.Selection, String])
], AIDiffWidget);
exports.AIDiffWidget = AIDiffWidget;
//# sourceMappingURL=inline-diff-widget.js.map