"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TerminalDetectionPromptManager = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const base_prompt_manager_1 = require("./base-prompt-manager");
let TerminalDetectionPromptManager = class TerminalDetectionPromptManager extends base_prompt_manager_1.BasePromptManager {
    generateBasePrompt(text) {
        const prompt = `When developing in the IDE, the terminal outputs some error messages. There may be multiple errors, and you need to provide a solution for each error. The error message is as follows: \`\`\`\n${text}\n\`\`\``;
        return prompt;
    }
    generateTsPrompt(text) {
        return `When developing in the IDE, the terminal outputs some error messages. There may be multiple errors, and you need to provide a solution for each error. The error message is as follows: \`\`\`\n${text}\n\`\`\``;
    }
    pickFileInfo(text) {
        const fileReg = /(?<path>[\w\/]+\.tsx?):(?<row>\d+):(?<col>\d+)/;
        const match = fileReg.exec(text);
        return match ? match.groups : undefined;
    }
    generateShellPrompt(stdout, stdin) {
        const inputPrompt = `Please provide a specific solution based on my input information: Input information: ${stdin}, `;
        const prompt = `An error occurred when entering commands in the terminal, ${stdin ? inputPrompt : 'please provide a possible solution'}, Error message: \`\`\`\n${stdout}\n\`\`\` `;
        return prompt;
    }
    generateJavaPrompt(text) {
        const errorTextArray = text.split('\n');
        // Cut off 10 lines of stack information, too much will cause the token to exceed the limit
        const errorText = errorTextArray.slice(0, 10).join('\n');
        const prompt = `The Java application has generated some errors during operation. Please provide a possible solution based on the error message. The error message is as follows: \`\`\`\n${errorText}\n\`\`\` `;
        return prompt;
    }
};
TerminalDetectionPromptManager = tslib_1.__decorate([
    (0, di_1.Injectable)()
], TerminalDetectionPromptManager);
exports.TerminalDetectionPromptManager = TerminalDetectionPromptManager;
//# sourceMappingURL=terminal-detection-prompt.js.map